from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.db.utils import IntegrityError
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _


class DirectPayment(models.Model):
    unique_identifier = models.CharField(
        max_length=255, blank=False, null=False, unique=True
    )
    status = models.CharField(
        max_length=255,
        choices=[
            (status, status) for status in settings.OSCAR_ORDER_STATUS_PIPELINE.keys()
        ],
    )
    callback_function = models.CharField(max_length=255, blank=False, null=False)

    def notify_callback(self, amount_paid):
        callback_function = import_string(self.callback_function)
        callback_function(self, self.status, amount_paid)

    @classmethod
    def new(cls, unique_identifier, callback_function, status, amount):
        try:
            instance = cls.objects.create(
                unique_identifier=unique_identifier,
                callback_function=callback_function,
                status=status,
            )
            instance.notify_callback(amount)
            return instance
        except IntegrityError:
            raise ValidationError(_("This payment already exists"))

    @classmethod
    def update_status(cls, unique_identifier, new_status, amount_paid):
        instance = cls.objects.get(unique_identifier=unique_identifier)

        if str(new_status) != str(instance.status):
            instance.status = new_status
            instance.save()

            instance.notify_callback(amount_paid)



from django import forms
from django.conf import settings
from django.contrib.auth.password_validation import validate_password
from django.utils.translation import gettext_lazy as _

from oscar.apps.checkout.forms import ShippingAddressForm as BaseShippingAddressForm
from oscar.apps.customer.forms import EmailUserCreationForm
from oscar.core.loading import get_model

from ocyan.core.fender import config

from .fields import MethodsChoiceField, StandardsCompliantRadioWidget

Country = get_model("address", "Country")


class ChooseShippingAddressForm(forms.Form):
    address_chooser = forms.ChoiceField(
        choices=[], required=False, label=_("Address chooser")
    )

    def __init__(self, request=None, default_address=None, *args, **kwargs):  # pylint: disable=W1113
        super().__init__(*args, **kwargs)
        if request is not None:
            user = request.user
            if user.is_authenticated:
                addresses = []

                for address in self.get_addresses(user):
                    addresses.append((address.pk, address))

                self.fields["address_chooser"].choices = addresses
                if default_address is not None:
                    self.fields["address_chooser"].initial = (
                        default_address["id"],
                        str(default_address),
                    )

    def get_addresses(self, user):
        return user.addresses.filter(country__is_shipping_country=True).order_by(
            "-is_default_for_shipping",
            "-num_orders_as_shipping_address",
            "-date_created",
        )


class ChooseBillingAddressForm(ChooseShippingAddressForm):
    def get_addresses(self, user):
        return user.addresses.order_by(
            "-is_default_for_billing", "-num_orders_as_billing_address", "-date_created"
        )


class PasswordCheckForm(forms.Form):
    password = forms.CharField(
        label=_("Password"), widget=forms.PasswordInput, required=False
    )
    password2 = forms.CharField(label=_("Confirm password"), widget=forms.PasswordInput)

    def clean_password2(self):
        password1 = self.cleaned_data.get("password", "")
        password2 = self.cleaned_data.get("password2", "")
        if password1 != password2:
            raise forms.ValidationError(_("The two password fields didn't match."))
        validate_password(password2)
        return password2

    def user_create_form(self, email):
        data = {
            "email": email,
            "password1": self.cleaned_data["password"],
            "password2": self.cleaned_data["password2"],
        }
        form = EmailUserCreationForm(data=data)
        if form.is_valid():
            return form
        for error in form.errors.values():
            self.add_error("__all__", error)
        return None


class SelectBillingAddressForm(forms.Form):
    require_billing_address = forms.BooleanField(
        label=_("Bill To Different Address"), required=False
    )


class ShippingMethodForm(forms.Form):
    method_code = MethodsChoiceField(
        label=_("Shipping method"), widget=StandardsCompliantRadioWidget()
    )

    def __init__(self, *args, **kwargs):
        self.methods = methods = kwargs.pop("methods", ())
        default = kwargs.pop("default", None)
        super(ShippingMethodForm, self).__init__(*args, **kwargs)
        method_code_field = self.fields["method_code"]
        method_code_field.initial = default
        method_code_field.methods = methods
        method_code_field.choices = ((m.code, m.name) for m in methods)
        if (  # if shipping has nothing to do with address, hide if possible
            config.get("shipping", "enable_free_shipping", False)
            and not (
                config.get("shipping", "enable_charged_shipping", False)
                or config.get("shipping", "enable_weightbased_shipping", False)
            )
            and len(methods) == 1
        ):
            method_code_field.widget = forms.HiddenInput()
            method_code_field.initial = methods[0].code

    @property
    def method(self):
        return next(
            (
                method
                for method in self.methods
                if self.cleaned_data["method_code"] == method.code
            )
        )


class ShippingAddressForm(BaseShippingAddressForm):
    field_order = settings.OSCAR_SHIPPING_FORM_FIELD_ORDER

    def adjust_country_field(self):
        countries = Country._default_manager.filter(  # pylint: disable=W0212
            is_shipping_country=True
        )

        # No need to show country dropdown if there is only one option
        if len(countries) == 1:
            self.fields["country"].widget = forms.HiddenInput()
            self.fields["country"].initial = self.instance.country = countries.get()
        else:
            self.fields["country"].queryset = countries
            self.fields["country"].empty_label = None

    class Meta(BaseShippingAddressForm.Meta):
        fields = settings.OSCAR_SHIPPING_FORM_FIELDS


class PaymentMethodForm(forms.Form):
    payment_method = forms.ChoiceField(choices={}, widget=forms.RadioSelect())

    def __init__(self, *args, **kwargs):
        payment_method_choices = kwargs.pop("payment_method_choices", None)
        super(PaymentMethodForm, self).__init__(*args, **kwargs)
        if payment_method_choices is not None:
            payment_method_field = self.fields["payment_method"]
            payment_method_field.choices = payment_method_choices

    @property
    def method(self):
        return self.cleaned_data["payment_method"]



# Generated by Django 2.0.13 on 2019-03-06 08:39

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [("address", "0006_auto_20181115_1953")]

    operations = [
        migrations.CreateModel(
            name="PaymentMethod",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("label", models.CharField(max_length=32)),
                ("key", models.CharField(max_length=32)),
                (
                    "country",
                    models.ManyToManyField(
                        to="address.Country", verbose_name="Countries"
                    ),
                ),
            ],
        )
    ]



from oscar.apps.checkout.applicator import (
    SurchargeApplicator as BaseSurchargeApplicator,
)
from oscar.core.loading import get_model

FixedSurcharge = get_model("checkout", "FixedSurcharge")
PercentageSurcharge = get_model("checkout", "PercentageSurcharge")
PaymentMethod = get_model("checkout", "PaymentMethod")


class SurchargeApplicator(BaseSurchargeApplicator):
    def get_surcharges(self, basket, **kwargs):
        surcharges = super(SurchargeApplicator, self).get_surcharges(basket)
        fixed_surcharges = FixedSurcharge.objects.all()
        percentage_surcharges = PercentageSurcharge.objects.all()
        return surcharges + tuple(fixed_surcharges) + tuple(percentage_surcharges)

    def is_applicable(self, surcharge, basket, **kwargs):
        if not isinstance(surcharge, (FixedSurcharge, PercentageSurcharge)):
            return True

        try:
            country = self.context["shipping_address"]["country"]
        except KeyError:
            country = None

        surcharge_countries = surcharge.country.all()
        payment_method_keys = surcharge.payment_method.all().values_list(
            "key", flat=True
        )

        if payment_method_keys:
            # Surchargemodel linked to the payment method
            payment_method = self.context.get("payment_method", "").strip()

            if country and surcharge_countries:
                return (
                    payment_method in payment_method_keys
                    and country in surcharge_countries
                )
            else:
                return payment_method in payment_method_keys

        if country and surcharge_countries:
            return country in surcharge_countries
        else:
            return True



import doctest

import ocyan.plugin.oscar_checkout.utils


def load_tests(loader, tests, ignore):  # pylint: disable=W0613
    tests.addTests(doctest.DocTestSuite(ocyan.plugin.oscar_checkout.utils))
    return tests



from oscar.defaults import OSCAR_REQUIRED_ADDRESS_FIELDS

from ocyan.core.fender import config

OCYAN_REST_FRAMEWORK_SETTINGS = {
    "REST_FRAMEWORK": {
        "DEFAULT_RENDERER_CLASSES": [
            "rest_framework.renderers.JSONRenderer",
            "rest_framework.renderers.BrowsableAPIRenderer",
        ],
        "DEFAULT_VERSIONING_CLASS": "rest_framework.versioning.NamespaceVersioning",
        "DEFAULT_PARSER_CLASSES": ["rest_framework.parsers.JSONParser"],
        "DEFAULT_AUTHENTICATION_CLASSES": [
            "rest_framework.authentication.SessionAuthentication"
        ],
    },
    "OSCARAPI_OVERRIDE_MODULES": [
        "ocyan.plugin.oscar_checkout.checkoutapi.api_extension"
    ],
    "OSCARAPI_ROOT_URLS": ["checkoutapi:api-root"],
}

# pylint: disable=no-member
OCYAN_CHECKOUT_OSCAR_SETTINGS = {
    "OSCAR_REQUIRED_ADDRESS_FIELDS": OSCAR_REQUIRED_ADDRESS_FIELDS
    + (
        "house_number",
        "email",
    ),
    "OSCARAPI_USERADDRESS_FIELDS": [
        "id",
        "email",
        "title",
        "first_name",
        "last_name",
        "line1",
        "line2",
        "line3",
        "line4",
        "state",
        "postcode",
        "search_text",
        "phone_number",
        "notes",
        "is_default_for_shipping",
        "is_default_for_billing",
        "country",
        "url",
        "house_number",
    ],
    "OSCAR_SHIPPING_FORM_FIELD_ORDER": [
        "title",
        "first_name",
        "last_name",
        "line1",
        "house_number",
        "line2",
        "line3",
        "line4",
        "state",
        "postcode",
        "country",
        "phone_number",
        "notes",
    ],
    "OSCAR_SHIPPING_FORM_FIELDS": [
        "title",
        "first_name",
        "last_name",
        "line1",
        "house_number",
        "line2",
        "line3",
        "line4",
        "state",
        "postcode",
        "country",
        "phone_number",
        "notes",
    ],
    "OSCAR_BILLING_FORM_FIELDS": [
        "email",
        "title",
        "first_name",
        "last_name",
        "line1",
        "house_number",
        "line2",
        "line3",
        "line4",
        "state",
        "postcode",
        "country",
    ],
    "OSCAR_BILLING_FORM_FIELD_ORDER": [
        "email",
        "title",
        "first_name",
        "last_name",
        "line1",
        "house_number",
        "line2",
        "line3",
        "line4",
        "state",
        "postcode",
        "country",
    ],
    "OSCAR_HIDDEN_FEATURES": (
        ["newsletter"] if "ocyan.plugin.newsletter" not in config.ocyan_plugins else []
    ),
}



# Generated by Django 2.2.13 on 2021-08-18 13:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0010_auto_20201117_1212'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='paymentmethod',
            options={'ordering': ['ordering', 'label']},
        ),
        migrations.AddField(
            model_name='paymentmethod',
            name='ordering',
            field=models.IntegerField(default=0, null=True),
        ),
    ]



# Generated by Django 2.0.13 on 2019-09-04 12:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymentmethod',
            name='requires_permission',
            field=models.BooleanField(default=False, help_text="Only users with the 'Allow <name of this payment method>' permission are allowed to use this paymentmethod"),
        ),
    ]



# Generated by Django 2.2.13 on 2020-11-17 11:12

from django.db import migrations
import oscar.models.fields.autoslugfield


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0009_fill_surcharge_code'),
    ]

    operations = [
        migrations.AlterField(
            model_name='surcharge',
            name='code',
            field=oscar.models.fields.autoslugfield.AutoSlugField(blank=True, editable=False, max_length=128, populate_from='name', unique=True, verbose_name='Code'),
        ),
    ]



# Generated by Django 4.2.11 on 2024-10-03 06:48

from django.db import migrations, models
from django.conf import settings

class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="DirectPayment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("unique_identifier", models.CharField(max_length=255, unique=True)),
                (
                    "status",
                    models.CharField(
                        choices=[(status, status) for status in settings.OSCAR_ORDER_STATUS_PIPELINE.keys()],
                        max_length=255,
                    ),
                ),
                ("callback_function", models.CharField(max_length=255)),
            ],
        ),
    ]



import copy
import logging
import operator

from django import http
from django.contrib import messages
from django.contrib.auth import login
from django.contrib.auth.mixins import UserPassesTestMixin
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.db import transaction
from django.forms.models import model_to_dict
from django.http import Http404
from django.shortcuts import get_object_or_404, redirect, render
from django.utils.functional import cached_property
from django.utils.http import urlsafe_base64_decode
from django.utils.translation import activate, gettext
from django.utils.translation import gettext_lazy as _
from django.views.generic import TemplateView
from rest_framework import response
from rest_framework.reverse import reverse
from rest_framework.settings import api_settings

from oscar.apps.checkout.exceptions import FailedPreCondition
from oscar.apps.checkout.views import ThankYouView as BaseThankYouView
from oscar.core.loading import feature_hidden, get_class, get_classes, get_model
from oscar.core.prices import Price

from oscarapi.signals import oscarapi_post_checkout

from ocyan.core.utils.cache import RequestCache
from ocyan.main.loading import get_ocyan_class
from ocyan.plugin.oscar.pipeline import (
    PIPELINE_STATUS_CANCELLED,
    PIPELINE_STATUS_NEW,
    PIPELINE_STATUS_PAID,
    PIPELINE_STATUS_PENDING_PAYMENT,
)

from ocyan.plugin.oscar_checkout.checkoutapi.serializers import PriceSerializer
from ocyan.plugin.oscar_checkout.utils import country_from_url

from .signals import newsletter_subscription

logger = logging.getLogger(__name__)

CheckoutSerializer = get_ocyan_class("checkoutapi.serializers", "CheckoutSerializer")
(
    payment_method_choices,
    get_available_shipping_methods,
    get_default_shipping_method,
) = get_classes(
    "checkout.utils",
    [
        "payment_method_choices",
        "get_available_shipping_methods",
        "get_default_shipping_method",
    ],
)

RegisterUserMixin = get_class("customer.mixins", "RegisterUserMixin")
ShippingAddressForm, ShippingMethodForm, GatewayForm = get_classes(
    "checkout.forms", ["ShippingAddressForm", "ShippingMethodForm", "GatewayForm"]
)
BillingAddressForm = get_class("payment.forms", "BillingAddressForm")
UserAddressForm = get_class("address.forms", "UserAddressForm")
PaymentMethodForm = get_class("checkout.forms", "PaymentMethodForm")
ChooseBillingAddressForm = get_class("checkout.forms", "ChooseBillingAddressForm")
ChooseShippingAddressForm = get_class("checkout.forms", "ChooseShippingAddressForm")
SelectBillingAddressForm = get_class("checkout.forms", "SelectBillingAddressForm")
PasswordCheckForm = get_class("checkout.forms", "PasswordCheckForm")
OrderTotalCalculator = get_class("checkout.calculators", "OrderTotalCalculator")
SurchargeApplicator = get_class("checkout.applicator", "SurchargeApplicator")
EventHandler = get_class("order.processing", "EventHandler")

Country = get_model("address", "Country")
UserAddress = get_model("address", "UserAddress")
Order = get_model("order", "Order")
Surcharge = get_model("order", "Surcharge")

BasketVoucherForm = get_class("basket.forms", "BasketVoucherForm")


ORDER_SESSION_KEY = "checkout_order_id"


class IndexView(RegisterUserMixin, TemplateView):
    KEY_FORMS = ["shipping_address_form", "shipping_method_form", "payment_method_form"]
    "These form are the ones that are REQUIRED to be able to fill an order"

    # A pre-condition is a condition that MUST be met in order for a view
    # to be available. If it isn't then the customer should be redirected
    # to a view *earlier* in the chain.
    # pre_conditions is a list of method names that get executed before the
    # normal flow of the view. Each method should check some condition has been
    # met. If not, then an exception is raised that indicates the URL the
    # customer will be redirected to.

    pre_conditions = ["check_basket_is_valid"]

    template_name = "checkout/onestep.html"

    def dispatch(self, request, *args, **kwargs):
        "Make sure that this view can correctly resolve urls for checkoutapi"
        # Enforce any pre-conditions for the view.
        try:
            self.check_pre_conditions(request)
        except FailedPreCondition as e:
            return http.HttpResponseRedirect(e.url)

        if api_settings.DEFAULT_VERSIONING_CLASS is not None:
            versioning_class = api_settings.DEFAULT_VERSIONING_CLASS
            request.version = "checkoutapi"
            request.versioning_scheme = versioning_class()

        return super(IndexView, self).dispatch(request, *args, **kwargs)

    @property
    def key_forms(self):
        return copy.copy(self.KEY_FORMS)

    def get_pre_conditions(self, request):
        """
        Return the pre-condition method names to run for this view
        """
        if self.pre_conditions is None:
            return []
        return self.pre_conditions

    def check_pre_conditions(self, request):
        pre_conditions = self.get_pre_conditions(request)
        for method_name in pre_conditions:
            if not hasattr(self, method_name):
                raise ImproperlyConfigured(
                    "There is no method '%s' to call as a pre-condition" % (method_name)
                )
            getattr(self, method_name)(request)

    def check_basket_is_valid(self, request):
        """
        Check that the basket is permitted to be submitted as an order. That
        is, all the basket lines are available to buy - nothing has gone out of
        stock since it was added to the basket.
        """
        strategy = request.strategy
        for line in request.basket.all_lines().select_related("stockrecord"):
            result = strategy.fetch_for_line(line, stockrecord=line.stockrecord)
            is_permitted = result.availability.is_purchase_permitted(line.quantity)[0]
            if not is_permitted:
                raise FailedPreCondition(url=reverse("basket:summary"))
        RequestCache.cache_for_request(request).clear()

    def forms(self, shipping_methods, shipping_method=None):
        """
        Create all the forms that are used in the template and populate them
        from the POST data if there is any
        """
        data = self.request.POST or None

        return {
            "gateway_form": GatewayForm(self.request, data, prefix="gateway_form"),
            "password_check_form": PasswordCheckForm(data, prefix="gateway_form"),
            "shipping_address_chooser_form": ChooseShippingAddressForm(
                self.request,
                self.get_default_shipping_address(),
                data,
                prefix="shipping_address_chooser",
            ),
            "billing_address_chooser_form": ChooseBillingAddressForm(
                self.request,
                self.get_default_billing_address(),
                data,
                prefix="billing_address_chooser",
            ),
            "shipping_method_form": ShippingMethodForm(
                data,
                methods=shipping_methods,
                default=shipping_method.code if shipping_method is not None else None,
                prefix="shipping_method_form",
            ),
            "shipping_address_form": ShippingAddressForm(
                data,
                initial=self.get_default_shipping_address(),
                prefix="shipping_address_form",
            ),
            "select_billing_address_form": SelectBillingAddressForm(
                data, prefix="select_billing_address_form"
            ),
            "billing_address_form": BillingAddressForm(
                data,
                initial=self.get_default_billing_address(),
                prefix="billing_address_form",
            ),
            "payment_method_form": PaymentMethodForm(
                data,
                prefix="payment_method_form",
                payment_method_choices=self.get_payment_methods(data),
            ),
        }

    def get_default_billing_address(self):
        user = self.request.user
        if user.is_authenticated:
            address = user.addresses.order_by(
                "-is_default_for_billing",
                "-num_orders_as_billing_address",
                "-date_created",
            ).first()
            if address is not None:
                return model_to_dict(address)

        return None

    def get_default_shipping_address(self):
        # Include only addresses where the country is flagged as valid for
        # shipping. Also, use ordering to ensure the default address comes
        # first.
        user = self.request.user
        if user.is_authenticated:
            address = (
                user.addresses.filter(country__is_shipping_country=True)
                .order_by(
                    "-is_default_for_shipping",
                    "-num_orders_as_shipping_address",
                    "-date_created",
                )
                .first()
            )
            if address is not None:
                address_dict = model_to_dict(address)
                if "notes" in address_dict:
                    del address_dict["notes"]
                return address_dict

        return None

    def show_shipping_address_chooser(self):
        if (
            self.request.user.is_authenticated
            and self.request.user.addresses.filter(
                country__is_shipping_country=True
            ).count()
            > 1
        ):
            return True

        return False

    def show_billing_address_chooser(self):
        if (
            self.request.user.is_authenticated
            and self.request.user.addresses.all().count() > 1
        ):
            return True

        return False

    def get_payment_methods(self, data=None):
        user = self.request.user
        basket = self.request.basket

        if data is not None:
            # find payment methods for selected country
            country_code = data.get("shipping_address_form-country")
            if country_code is not None:
                return payment_method_choices(user, basket, country__pk=country_code)
        elif (
            self.request.user.is_authenticated
        ):  # no data? try to fill from existing user address
            default_address = self.get_default_shipping_address()
            if default_address is not None:
                return payment_method_choices(
                    user, basket, country__pk=default_address["country"]
                )

        # get the default country and it's payment choices!
        return payment_method_choices(
            user,
            basket,
            country=Country.objects.filter(is_shipping_country=True).first(),
        )

    def get_context_data(self, shipping_method=None, **kwargs):  # pylint: disable=W0221
        """
        Add the template requirements to the context
        """
        ctx = super(IndexView, self).get_context_data(**kwargs)

        request = self.request
        basket = request.basket
        shipping_address = self.get_default_shipping_address()
        shipping_methods = get_available_shipping_methods(request, shipping_address)
        shipping_method = (
            shipping_method
            if shipping_method is not None
            else get_default_shipping_method(request, shipping_address)
        )
        shipping_charge = shipping_method.calculate(basket)

        ctx.update(self.forms(shipping_methods, shipping_method))
        ctx.update(
            {
                "shipping_methods": shipping_methods,
                "shipping_method": shipping_method,
                "shipping_charge": shipping_charge,
                "basket": basket,
                "voucher_form": BasketVoucherForm(),
                "show_shipping_address_chooser": self.show_shipping_address_chooser(),
                "show_billing_address_chooser": self.show_billing_address_chooser(),
            }
        )
        ctx.update(kwargs)
        surcharges = SurchargeApplicator(self.request, ctx).get_applicable_surcharges(
            basket
        )
        order_total = OrderTotalCalculator().calculate(
            basket, shipping_charge, surcharges
        )
        ctx["order_total"] = order_total
        ctx["surcharges"] = surcharges
        return ctx

    def get_address_data(self, forms, key="shipping_address_form"):
        address_form = forms[key]
        if address_form.is_valid():
            address = address_form.cleaned_data
            address["country"] = reverse(
                "country-detail", args=(address["country"].pk,), request=self.request
            )
            return address

        return None

    def get_shipping_address_data(self, forms):
        return self.get_address_data(forms, "shipping_address_form")

    def get_billing_address_data(self, forms):
        return self.get_address_data(forms, "billing_address_form")

    def build_serializer_data(self, forms):
        """
        Because we use the oscarapi serializer to create the order we have to
        do a little bit of processing to turn the validated form data into
        the serializers data structure.
        """
        request = self.request
        basket = request.basket
        shipping_method_form = forms["shipping_method_form"]
        payment_method_form = forms["payment_method_form"]
        shipping_method = shipping_method_form.method
        shipping_address = self.get_shipping_address_data(forms)

        shipping_country = country_from_url(shipping_address.get("country", None))

        request.META["shipping_country"] = shipping_country

        serializer_data = {
            "basket": reverse("basket-detail", args=(basket.pk,), request=request),
            "shipping_address": shipping_address,
            "shipping_method_code": shipping_method_form.cleaned_data["method_code"],
            "shipping_charge": PriceSerializer(shipping_method.calculate(basket)).data,
            "payment_method": payment_method_form.method,
        }

        if self.require_billing_address(forms):
            billing_address = self.get_billing_address_data(forms)
            if billing_address is not None:
                serializer_data["billing_address"] = billing_address

        if (
            not request.user.is_authenticated
            and forms["gateway_form"].is_guest_checkout()
        ):
            serializer_data["guest_email"] = forms["gateway_form"].cleaned_data[
                "username"
            ]

        surcharges = SurchargeApplicator(
            request, serializer_data
        ).get_applicable_surcharges(basket)

        serializer_data["surcharges"] = surcharges

        return serializer_data

    def require_billing_address(self, forms):
        select_billing_address_form = forms["select_billing_address_form"]
        return (
            select_billing_address_form.is_valid()
            and select_billing_address_form.cleaned_data["require_billing_address"]
        )

    def key_forms_valid(self, forms):
        """
        Returns ``True`` if all of the forms required for the order are valid.
        """
        key_forms = self.key_forms
        if self.require_billing_address(forms):
            key_forms.append("billing_address_form")

        select_key_forms = operator.itemgetter(*key_forms)

        # If one of the key forms is invalid, return False
        return not any([not x.is_valid() for x in select_key_forms(forms)])

    def handle_gateway_form(self, gateway_form, password_check_form):
        """
        Register or log in user as specified by the options input.
        """
        gateway_form_valid = gateway_form.is_valid()

        if self.request.user.is_authenticated:
            return True, False
        elif not gateway_form_valid:
            return False, False

        if gateway_form.is_guest_checkout():
            return True, False
        elif gateway_form.is_new_account_checkout():
            if not password_check_form.is_valid():
                return False, False

            email = gateway_form.cleaned_data["username"]
            create_form = password_check_form.user_create_form(email)
            if create_form is None:
                return False, False
            # since the user is new, it is safe to assign the anonymous
            # basket to the new user without creating a new basket.
            self.request.basket.owner = self.register_user(create_form)
            self.request.basket.save()
            return True, False
        else:
            user = gateway_form.get_user()
            login(self.request, user)
            return True, True

    def handle_newsletter_subscription(self, POST, order, user):
        if not feature_hidden("newsletter") and POST.get("subscribetonewsletter"):
            try:
                full_name = order.shipping_address.name
            except AttributeError:
                if user.is_authenticated:
                    full_name = user.get_full_name()
                else:
                    full_name = "unknown"

            newsletter_subscription.send(
                sender=order.__class__, full_name=full_name, email=order.email
            )

    def handle_terms_of_service(self, request, gateway_passed):
        if not request.POST.get("accepttermsofservice"):
            gateway_passed = False
            messages.error(request, gettext("Please accept our terms of service."))

        return gateway_passed

    def get_posted_shipping_address(self, request):
        """
        Get the posted shipping adddres before validating all the forms.
        """
        forms = self.forms([])
        shipping_address_form = forms["shipping_address_form"]
        if shipping_address_form.is_valid():
            return shipping_address_form.cleaned_data
        else:
            return shipping_address_form.cleaned_data

    def get(self, request, *args, **kwargs):
        if request.basket.is_empty and ORDER_SESSION_KEY in request.session:
            order = Order.objects.get(pk=request.session[ORDER_SESSION_KEY])
            if order.status in [PIPELINE_STATUS_NEW, PIPELINE_STATUS_PENDING_PAYMENT]:
                messages.error(
                    request,
                    gettext(
                        "Something went wrong while placing your order, if you haven't paid yet, you can do so on this page. In case you did make payment, please contact our customer service for assistance."
                    ),
                )
                return redirect("checkout:try-again")

        return super().get(request, *args, **kwargs)

    def post(self, request):
        """
        Handle the creation of the order.
        """
        request.META["shipping_country"] = request.POST.get(
            "shipping_address_form-country"
        )
        shipping_address = self.get_posted_shipping_address(request)
        shipping_methods = get_available_shipping_methods(request, shipping_address)

        basket = request.basket
        context = {}

        forms = self.forms(shipping_methods)
        gateway_passed, user_logged_in = self.handle_gateway_form(
            forms["gateway_form"], forms["password_check_form"]
        )

        # if we login using gateway_form redirect to checkout index.
        if user_logged_in:
            return redirect("checkout:index")

        # Handle edge cases where something happened to the connection while
        # placing an order, in that case the basket should be empty and
        # a ORDER_SESSION_KEY should be found in the session
        if basket.is_empty and ORDER_SESSION_KEY in request.session:
            order = Order.objects.get(pk=request.session[ORDER_SESSION_KEY])
            if order.status == PIPELINE_STATUS_NEW:
                # if the status of the order is PIPELINE_STATUS_NEW and the
                # payment_method info was posted again for the second time by
                # the client, we can just redirect to the correct payment method.
                if forms["payment_method_form"].is_valid():
                    return redirect(forms["payment_method_form"].method)

            # if the order had one of these 2 statusses the client can retry
            # payment on the try again view.
            if order.status in [PIPELINE_STATUS_NEW, PIPELINE_STATUS_PENDING_PAYMENT]:
                messages.error(
                    request,
                    gettext(
                        "Something went wrong while placing your order, if you haven't paid yet, you can do so on this page. In case you did make payment, please contact our customer service for assistance."
                    ),
                )
                return redirect("checkout:try-again")

            if order.status == PIPELINE_STATUS_PAID:
                return redirect("checkout:thank-you")

            # if we got here, there is nothing wrong so let's ignore.

        gateway_passed = self.handle_terms_of_service(request, gateway_passed)

        if self.key_forms_valid(forms) and gateway_passed:
            serializer_data = self.build_serializer_data(forms)
            ser = CheckoutSerializer(data=serializer_data, context={"request": request})

            if ser.is_valid():
                basket.freeze()
                order = ser.save()
                oscarapi_post_checkout.send(
                    sender=self,
                    order=order,
                    user=request.user,
                    request=request,
                    response=response,
                )
                self.handle_newsletter_subscription(request.POST, order, request.user)
                request.session[ORDER_SESSION_KEY] = order.id
                return redirect(forms["payment_method_form"].method)
            else:
                raise ValidationError(
                    "Serializer populated from validated form has errors %s"
                    % ser.errors
                )
        else:
            shipping_method_form = forms["shipping_method_form"]
            if shipping_method_form.is_valid():
                context["shipping_method"] = shipping_method_form.method

        for _, form in forms.items():
            for error in form.non_field_errors():
                messages.error(request, error)

        context.update(forms)
        context = self.get_context_data(**context)
        return self.render_to_response(context)


class ThankYouView(BaseThankYouView):
    def get_object(self, *args, **kwargs):
        order = super().get_object(*args, **kwargs)
        if order is not None:
            activate(order.language_code)
        return order


class ThankYouFromLinkView(ThankYouView):
    def get(self, request, oidb64, token):  # pylint: disable=W0221
        self.object = self.get_object(oidb64, token)  # pylint: disable=W0201
        if self.object is None:
            return redirect("checkout:order-not-found")

        return self.render_to_response(self.get_context_data(order=self.object))

    def get_object(self, oidb64, token):  # pylint: disable=W0221
        try:
            order_id = urlsafe_base64_decode(oidb64).decode()
        except UnicodeDecodeError:
            return None

        order = Order.objects.filter(pk=order_id).first()
        if order is None:
            return None

        if not order.check_verification_hash(token):
            return None

        return order


class OrderNotFoundView(TemplateView):
    template_name = "checkout/order_not_found.html"


class CancelView(ThankYouView):
    template_name = "checkout/cancel.html"

    def get(self, request, *args, **kwargs):
        order = self.get_object()
        if order.status == PIPELINE_STATUS_CANCELLED:
            request.basket.save()  # basket could be newly created, save first.
            request.basket.merge(order.basket)

        return super(CancelView, self).get(request, *args, **kwargs)


class TryAgainView(TemplateView):
    template_name = "checkout/try_again.html"

    def get(self, request, *args, **kwargs):
        if (
            self.order.status == PIPELINE_STATUS_PENDING_PAYMENT
            or self.order.status == PIPELINE_STATUS_NEW
        ):
            # Check if javascript request for partial update regarding totals
            if request.headers.get("X-Requested-With") == "XMLHttpRequest":
                return self.render_totals(request, *args, **kwargs)

            return super().get(request, *args, **kwargs)
        else:
            raise Http404

    def post(self, request, *args, **kwargs):
        data = request.POST or None
        form = self.payment_method_form(data)
        event_handler = EventHandler(user=request.user)

        if request.POST.get("_cancel"):
            event_handler.handle_order_status_change(
                self.order, PIPELINE_STATUS_CANCELLED
            )
            return redirect("checkout:cancel")

        if request.POST.get("_try-again"):
            if form.is_valid():
                # The order could have changed (eg; surcharges from the chosen payment method),
                # make sure to save it before redirecting to the payment gateway (save=True).
                self.get_recalculated_order_obj(
                    surcharges=self.get_surcharges(form.cleaned_data["payment_method"]),
                    save=True,
                )
                return redirect(form.method)
            else:
                messages.error(request, gettext("Select a payment method"))
                return render(
                    request, self.template_name, context=self.get_context_data()
                )

        # Invalid POST request, return 404.
        raise Http404

    def render_totals(self, request, *args, **kwargs):
        payment_method = request.GET.get("payment_method", "")
        context = self.get_order_context(payment_method)
        context.update(
            {
                "nohide": True,
                "title": gettext("Your order"),
                "id": "checkouttotal",
            }
        )
        return render(request, "checkout/partials/try_again_totals_block.html", context)

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx["payment_method_form"] = self.payment_method_form()
        # make sure nobody can mess with the basket
        ctx["basket"] = None
        ctx.update(self.get_order_context())
        return ctx

    def get_order_context(self, payment_method=""):
        surcharges = self.get_surcharges(payment_method)
        recalculated_order = self.get_recalculated_order_obj(surcharges)
        return {
            "order": recalculated_order,
            "order_total": Price(
                currency=recalculated_order.currency,
                excl_tax=recalculated_order.total_excl_tax,
                incl_tax=recalculated_order.total_incl_tax,
            ),
            "surcharges": surcharges,
        }

    def payment_method_form(self, data=None):
        user = self.request.user
        choices = payment_method_choices(
            user, self.order, country=self.order.shipping_address.country
        )
        return PaymentMethodForm(data, payment_method_choices=choices)

    @cached_property
    def order(self):
        if ORDER_SESSION_KEY in self.request.session:
            order = get_object_or_404(
                Order, pk=self.request.session[ORDER_SESSION_KEY], basket__isnull=False
            )
            order.basket.strategy = self.request.basket.strategy
            return order

        raise Http404(gettext("No order found"))

    @cached_property
    def order_shipping_charge(self):
        return Price(
            currency=self.order.currency,
            excl_tax=self.order.shipping_excl_tax,
            incl_tax=self.order.shipping_incl_tax,
        )

    def get_try_again_basket(self):
        """
        Returns a basket that's compatible with the TryAgain logic. It returns
        the order's basket totals, instead of the total of the basket itself. The basket
        itself recalculates all lines, and for now, we don't implement this as this would
        require updating A LOT of fields, related models (line price, line discounts etc)
        and all of that has to be done without losing the original order.
        """

        class TryAgainCompatibleBasket:
            def __init__(self, basket, order):
                self.basket = basket
                self.order = order

            def __getattr__(self, name):
                return getattr(self.basket, name)

            @property
            def total_excl_tax(self):
                return self.order.basket_total_excl_tax

            @property
            def total_incl_tax(self):
                return self.order.basket_total_incl_tax

        return TryAgainCompatibleBasket(self.order.basket, self.order)

    def get_surcharges(self, payment_method=""):
        context = {
            "payment_method": payment_method,
            "shipping_address": self.order.shipping_address.__dict__,
        }
        return SurchargeApplicator(self.request, context).get_applicable_surcharges(
            self.get_try_again_basket(), shipping_charge=self.order_shipping_charge
        )

    def recalculate_order_total(self, surcharges=None):
        """
        This method recalculates the order total based on the NEW surcharges.
        We can NOT use the OrderTotalCalculator, as this also re-calculates the
        basket lines, which we don't want.
        """
        total_without_surcharge = Price(
            currency=self.order.currency,
            excl_tax=self.order.total_excl_tax - self.order.surcharge_excl_tax,
            incl_tax=self.order.total_incl_tax - self.order.surcharge_incl_tax,
        )

        if surcharges:
            total_without_surcharge += surcharges.total

        return total_without_surcharge

    def get_recalculated_order_obj(self, surcharges, save=False):
        new_order_total = self.recalculate_order_total(surcharges)

        self.order.total_incl_tax = new_order_total.incl_tax
        self.order.total_excl_tax = new_order_total.excl_tax

        # If save has been set to True, we should save the order with the new totals.
        # We should also create surcharge models for the order.
        if save:
            with transaction.atomic():
                # First, delete the old surcharges
                self.order.surcharges.all().delete()

                # Then, create the new surcharges
                if surcharges:
                    for charge in surcharges:
                        surcharge = Surcharge(
                            order=self.order,
                            name=charge.surcharge.name,
                            code=charge.surcharge.code,
                            excl_tax=charge.price.excl_tax,
                            incl_tax=charge.price.incl_tax,
                        )
                        # check if the surcharge model has a 'tax_code' field (compatibility with older versions of Oscar)
                        if hasattr(surcharge, "tax_code"):
                            surcharge.tax_code = charge.price.tax_code

                        surcharge.save()

                self.order.save()

        return self.order


class TryAgainFromLinkView(UserPassesTestMixin, TryAgainView):
    template_name = "checkout/try_again_from_link.html"

    def test_func(self):
        order = self.order

        if order.is_anonymous:
            if order.check_verification_hash(self.kwargs["token"]):
                return True

            # pretend this page does not exist if verification_hash check
            # failed
            raise Http404()

        if not self.request.user.is_authenticated:
            # redirect to login if user is nog logged in and order is
            # not anynymous
            return False

        if order.user is not None and self.request.user != order.user:
            # pretend page does not exist if user tries to access order
            # from a different user
            raise Http404()

        if not order.check_verification_hash(self.kwargs["token"]):
            # if the token is not valid, pretend the page does not exist
            raise Http404()

        return True

    @cached_property
    def order(self):
        order_id = urlsafe_base64_decode(self.kwargs["oidb64"]).decode()
        order = get_object_or_404(Order, pk=order_id, basket__isnull=False)
        order.basket.strategy = self.request.basket.strategy
        return order

    def post(self, request, *args, **kwargs):
        resp = super().post(request, *args, **kwargs)
        try:
            if resp.status_code == 302:
                if not request.POST.get("_cancel"):
                    request.session[ORDER_SESSION_KEY] = self.order.pk
        except (AttributeError, IndexError):
            pass

        return resp



# Generated by Django 2.0.13 on 2019-10-30 14:18

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0002_paymentmethod_requires_permission'),
    ]

    operations = [
        migrations.CreateModel(
            name='Surcharge',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=32)),
                ('excl_tax', models.DecimalField(blank=True, decimal_places=2, max_digits=12, null=True)),
                ('incl_tax', models.DecimalField(blank=True, decimal_places=2, max_digits=12, null=True)),
                ('percentage', models.DecimalField(blank=True, decimal_places=2, max_digits=12, null=True)),
                ('is_percentage', models.BooleanField(default=False)),
            ],
        ),
        migrations.AddField(
            model_name='paymentmethod',
            name='surcharge',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='checkout.Surcharge'),
        ),
    ]


# Generated by Django 3.2.13 on 2022-06-07 11:23

from django.db import migrations
from django.db.models import Q


def forwards(apps, schema_editor):
    PaymentMethod = apps.get_model("checkout", "PaymentMethod")
    Surcharge = apps.get_model("checkout", "Surcharge")
    PercentageSurcharge = apps.get_model("checkout", "PercentageSurcharge")
    FixedSurcharge = apps.get_model("checkout", "FixedSurcharge")

    Surcharge.objects.filter(
        Q(is_percentage=True, percentage__isnull=True)
        | Q(is_percentage=False, excl_tax__isnull=True)
        | Q(is_percentage=False, incl_tax__isnull=True)
    ).delete()

    for surcharge in Surcharge.objects.filter(is_percentage=True):
        new_surcharge, _ = PercentageSurcharge.objects.get_or_create(
            code=surcharge.code,
            name=surcharge.name,
            percentage=surcharge.percentage,
            include_shipping_charges=surcharge.include_shipping_charges,
        )
        payment_methods = PaymentMethod.objects.filter(surcharge=surcharge)
        if payment_methods:
            new_surcharge.payment_method.add(*payment_methods)

    for surcharge in Surcharge.objects.filter(is_percentage=False):
        new_surcharge, _ = FixedSurcharge.objects.get_or_create(
            code=surcharge.code,
            name=surcharge.name,
            excl_tax=surcharge.excl_tax,
            incl_tax=surcharge.incl_tax,
        )
        payment_methods = PaymentMethod.objects.filter(surcharge=surcharge)
        if payment_methods:
            new_surcharge.payment_method.add(*payment_methods)



class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0012_fixedsurcharge_percentagesurcharge'),
    ]

    operations = [
        migrations.RunPython(forwards, migrations.RunPython.noop),
    ]



from django.urls import path, re_path
from rest_framework.urlpatterns import format_suffix_patterns

# pylint: disable=no-name-in-module
from oscar.core.application import OscarConfig

from oscarapi.utils.loading import get_api_class, get_api_classes


class Oscar3ApiAppConfig(OscarConfig):
    name = "ocyan.plugin.oscar_checkout.checkoutapi"
    label = "checkoutapi"
    verbose_name = "Oscar api"
    namespace = "checkoutapi"

    def get_urls(self):
        from . import views

        LoginView = get_api_class("views.login", "LoginView")

        # pylint: disable=unbalanced-tuple-unpacking
        (BasketDetail, CountryList, CountryDetail) = get_api_classes(
            "views.basic", ["BasketDetail", "CountryList", "CountryDetail"]
        )

        UserAddressDetail = get_api_class("views.checkout", "UserAddressDetail")

        return format_suffix_patterns(
            [
                re_path(r"^$", views.api_root, name="api-root"),
                re_path(
                    r"^basket/shipping-methods/$",
                    views.ShippingMethodView.as_view(),
                    name="api-basket-shipping-methods",
                ),
                re_path(
                    r"^basket/total/$",
                    views.OrderTotalView.as_view(),
                    name="api-basket-total",
                ),
                re_path(
                    r"^basket/payment-methods/$",
                    views.PaymentMethodView.as_view(),
                    name="api-basket-payment-methods",
                ),
                path("login/", LoginView.as_view(), name="api-login"),
                path("baskets/<int:pk>/", BasketDetail.as_view(), name="basket-detail"),
                path("countries/", CountryList.as_view(), name="country-list"),
                re_path(
                    r"^countries/(?P<pk>[A-z]{2})/$",
                    CountryDetail.as_view(),
                    name="country-detail",
                ),
                path(
                    "useraddresses/<int:pk>/",
                    UserAddressDetail.as_view(),
                    name="useraddress-detail",
                ),
            ]
        )



from collections import OrderedDict

from phonenumber_field.phonenumber import PhoneNumber
from rest_framework import serializers

from oscar.forms.mixins import PhoneNumberMixin


class PhoneNumberFormContext(PhoneNumberMixin):
    """
    This class created a context the PhoneNumberMixin can function in, so it
    can be used with a serializer.

    It sets up the variables ``cleaned_data``, ``region``, ``country`` and
    provides and add_error function.
    """

    def add_error(self, field_name, error):
        errors = OrderedDict()
        errors[field_name] = error
        raise serializers.ValidationError(errors)

    def __init__(self, data):
        # don't call super, it does some shit we don't want
        # pylint: disable=super-init-not-called

        if "phone_number" in data:
            phone_number = data["phone_number"]

            # Newer versions of django-phonenumber-field has implemented to_python on the model field
            # which returns the PhoneNumber object, but we use this class in .serialziers.py to call the
            # clean_phone_number_field method which expects to work with a string.
            if isinstance(phone_number, PhoneNumber):
                data["phone_number"] = phone_number.raw_input

        self.cleaned_data = data
        self.country = data.get("country", None)
        # 3. PhoneNumberMixin uses region_code
        if self.country is not None:
            self.region_code = self.country.iso_3166_1_a2



from decimal import Decimal as D

from django.db import models
from django.utils.translation import gettext_lazy as _

from oscar.core import prices
from oscar.core.loading import get_class, get_classes
from oscar.models.fields import AutoSlugField

from ocyan.core.fender import config
from ocyan.core.utils import round_to_two_dec
from ocyan.plugin.oscar.constants import TAX_RATE_ZERO

IncludingTaxPriceSelector = get_class("partner.taxbase", "IncludingTaxPriceSelector")
ExcludingTaxPriceSelector = get_class("partner.taxbase", "ExcludingTaxPriceSelector")

taxed_shipping_charge = get_class("shipping.utils", "taxed_shipping_charge")

tax_rate_as_fraction, get_default_tax_rate, ALL_TAX_RATES = get_classes(
    "taxes.taxrate",
    [
        "tax_rate_as_fraction",
        "get_default_tax_rate",
        "ALL_TAX_RATES",
    ],
)

USE_PRICE_INCL_TAX = config.get("oscar", "use_price_incl_tax", True)


class PaymentMethod(models.Model):
    country = models.ManyToManyField("address.Country", verbose_name=_("Countries"))
    label = models.CharField(max_length=32)
    key = models.CharField(max_length=32)
    send_invoice = models.BooleanField(
        default=config.get("oscar_communications", "invoice", False)
    )
    requires_permission = models.BooleanField(
        default=False,
        help_text="Only users with the 'Allow <name of this payment method>' permission are allowed to use this paymentmethod",
    )
    min_amount = models.DecimalField(
        _("Minimum basket amount"),
        decimal_places=2,
        max_digits=12,
        default=D("0.00"),
        help_text=_("The minimum amount before this payment is available"),
    )
    max_amount = models.DecimalField(
        _("Maximum basket amount"),
        decimal_places=2,
        max_digits=12,
        null=True,
        blank=True,
        help_text=_(
            "Some payment methods like pay after have a maximum amount to protect the merchant. If the basket contains more then this amount, the payment method is not available"
        ),
    )
    ordering = models.IntegerField(null=True, default=0)

    def has_permission(self, user):
        return not self.requires_permission or user.has_perm("checkout.%s" % self.key)

    def __str__(self):
        return self.label

    class Meta:
        ordering = ["ordering", "label"]


class AbstractSurcharge(models.Model):
    name = models.CharField(max_length=32, null=False)
    code = AutoSlugField(_("Code"), max_length=128, unique=True, populate_from="name")
    country = models.ManyToManyField(
        "address.Country", blank=True, verbose_name=_("Countries")
    )
    payment_method = models.ManyToManyField(
        "checkout.PaymentMethod",
        blank=True,
        verbose_name=_("Payment methods"),
    )
    vat_rate = models.CharField(
        _("VAT rate"),
        max_length=5,
        choices=ALL_TAX_RATES,
        default=get_default_tax_rate(),
        null=True,
    )

    def __str__(self):
        return self.name

    class Meta:
        abstract = True


class PercentageSurcharge(AbstractSurcharge):
    percentage = models.DecimalField(
        decimal_places=2, max_digits=12, null=False, blank=False
    )
    include_shipping_charges = models.BooleanField(default=True)

    def calculate(self, basket, **kwargs):
        if not basket.is_empty:
            shipping_charge = kwargs.get("shipping_charge", None)

            if shipping_charge is not None and self.include_shipping_charges:
                total_incl_tax = basket.total_incl_tax + shipping_charge.incl_tax
            else:
                total_incl_tax = basket.total_incl_tax

            surcharge_incl_tax = round_to_two_dec(
                total_incl_tax * (self.percentage / D(100))
            )

            price = IncludingTaxPriceSelector(
                basket.owner,
                basket.strategy.request,
                currency=basket.currency,
                incl_tax=surcharge_incl_tax,
                vat_rate=self.vat_rate,
            )
        else:
            price = IncludingTaxPriceSelector(
                basket.owner,
                basket.strategy.request,
                currency=basket.currency,
                incl_tax=D(0),
                vat_rate=TAX_RATE_ZERO,
            )

        kwargs = {
            "currency": price.currency,
            "excl_tax": price.excl_tax,
            "incl_tax": price.incl_tax,
            "tax_code": price.tax_code,
        }

        return prices.Price(**kwargs)


class FixedSurcharge(AbstractSurcharge):
    price = models.DecimalField(
        decimal_places=2, max_digits=12, null=False, blank=False
    )

    def calculate(self, basket, **kwargs):
        if USE_PRICE_INCL_TAX:
            price = IncludingTaxPriceSelector(
                basket.owner,
                basket.strategy.request,
                currency=basket.currency,
                incl_tax=self.price,
                vat_rate=self.vat_rate,
            )
        else:
            price = ExcludingTaxPriceSelector(
                basket.owner,
                basket.strategy.request,
                currency=basket.currency,
                excl_tax=self.price,
                vat_rate=self.vat_rate,
            )

        kwargs = {
            "currency": price.currency,
            "excl_tax": price.excl_tax,
            "incl_tax": price.incl_tax,
            "tax_code": price.tax_code,
        }

        return prices.Price(**kwargs)


from oscar.apps.checkout.models import *  # noqa



# Generated by Django 2.0.13 on 2019-10-31 11:03

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0003_auto_20191030_1418'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentmethod',
            name='surcharge',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='checkout.Surcharge'),
        ),
    ]



# Generated by Django 2.2.13 on 2020-08-18 09:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0006_paymentmethod_send_invoice'),
    ]

    operations = [
        migrations.AddField(
            model_name='surcharge',
            name='include_shipping_charges',
            field=models.BooleanField(default=True),
        ),
    ]



# Generated by Django 2.2.13 on 2020-11-17 11:07

from django.db import migrations


def forward(apps, schema_editor):
    Surcharge = apps.get_model("checkout", "Surcharge")
    for s in Surcharge.objects.all():
        s.save()


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0008_surcharge_code'),
    ]

    operations = [
        migrations.RunPython(forward, migrations.RunPython.noop),
    ]



# Generated by Django 2.0.13 on 2019-12-03 08:45

from decimal import Decimal
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0004_auto_20191031_1103'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymentmethod',
            name='max_amount',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='Some payment methods like pay after have a maximum amount to protect the merchant. If the basket contains more then this amount, the payment method is not available', max_digits=12, null=True, verbose_name='Maximum basket amount'),
        ),
        migrations.AddField(
            model_name='paymentmethod',
            name='min_amount',
            field=models.DecimalField(decimal_places=2, default=Decimal('0.00'), help_text='The minimum amount before this payment is available', max_digits=12, verbose_name='Minimum basket amount'),
        ),
        migrations.AlterField(
            model_name='paymentmethod',
            name='surcharge',
            field=models.ForeignKey(blank=True, null=True, on_delete=models.deletion.CASCADE, to='checkout.Surcharge', verbose_name='Surcharge'),
        ),
    ]



from django.test import TestCase

from oscar.core.loading import get_model

Order = get_model("order", "Order")


class CheckoutThankYouTest(TestCase):
    fixtures = [
        "checkout/auth",
        "checkout/catalogue",
        "checkout/shipping",
        "checkout/basket",
        "checkout/order",
    ]

    def test_thankyou_from_link_view(self):
        order = Order.objects.get(pk="1")
        resp = self.client.get(order.get_thank_you_url(), follow=True)

        self.assertEqual(resp.context.get("object", None), order)
        self.assertEqual(resp.template_name[0], "oscar/checkout/thank_you.html")
        self.assertEqual(
            resp.context["language_neutral_url_path"],
            "/checkout/thank-you/MQ/hoela202100000001:m8SxZrjG1ERKfA-NRgW4tfzP-aqcttY1WMAfNuGW7e4/",
        )

    def test_thankyou_from_link_invalid_token(self):
        resp = self.client.get("/checkout/thank-you/MQ/fail/")
        self.assertEqual(resp.url, "/checkout/order-not-found/")
        self.assertEqual(resp.status_code, 302)

    def test_thankyou_from_link_invalid_oidb64(self):
        resp = self.client.get(
            "/checkout/thank-you/fail/hoela202100000001:m8SxZrjG1ERKfA-NRgW4tfzP-aqcttY1WMAfNuGW7e4/"
        )
        self.assertEqual(resp.url, "/checkout/order-not-found/")
        self.assertEqual(resp.status_code, 302)

    def test_thankyou_from_link_invalid_no_oidb64_or_token(self):
        resp = self.client.get("/checkout/thank-you/", follow=True)

        self.assertEqual(resp.status_code, 200)



# Generated by Django 3.2.13 on 2022-06-07 11:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0013_surcharge_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='paymentmethod',
            name='surcharge',
        ),
        migrations.DeleteModel(
            name='Surcharge',
        ),
    ]



import re

from oscar.core.decorators import deprecated
from oscar.core.loading import get_model

from ocyan.plugin.oscar.utils import set_user_properties

COUNTRY_REGEX = re.compile(r"countries/(?P<country_code>\w+)/$")

PaymentMethod = get_model("checkout", "PaymentMethod")


def country_from_url(url):
    """
    Parse a url for the country code

    >>> country_from_url("klaas")
    >>> country_from_url("http://example.com/countries/YUBA/")
    'YUBA'
    >>> country_from_url("http://example.com/countries/YUBA/Lu")
    >>> country_from_url("http://example.com/countries/nl")
    >>> country_from_url("http://example.com/countries/nl/")
    'nl'
    """
    country = COUNTRY_REGEX.search(url)
    if country is not None:
        return country.group("country_code")

    return None


set_user_properties = deprecated(set_user_properties)



from decimal import Decimal as D
from unittest import mock

from django.contrib.auth import get_user_model
from django.test import TestCase
from django.urls import reverse

from oscar.core.loading import get_model

import ocyan.core.fender
from ocyan.core.utils import merge_dicts

User = get_user_model()
Basket = get_model("basket", "Basket")
PaymentMethod = get_model("checkout", "PaymentMethod")
ocyan_settings = ocyan.core.fender.config.settings
CHECKOUT_OCYAN_OPTIONS = merge_dicts(
    ocyan_settings, {"shipping": {"enable_charged_shipping": True}}
)


@mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUT_OCYAN_OPTIONS)
class CheckoutPaymentMethodTest(TestCase):
    fixtures = [
        "checkout/auth",
        "checkout/catalogue",
        "checkout/shipping",
        "checkout/paymentmethod",
    ]

    @mock.patch("sorl.thumbnail.templatetags.thumbnail.ThumbnailNodeBase.render")
    def setup_checkout(self, render):
        render.return_value = ""
        henk = User.objects.get(username="henk")
        self.client.force_login(henk)
        self.assertEqual(Basket.objects.count(), 0, "The basket should be empty")

        basket_add_url = reverse("basket:add", args=(1,))
        body = {"quantity": 3}

        response = self.client.post(basket_add_url, body)
        self.assertEqual(
            response.status_code, 302, "Adding an item to a basket should redirect."
        )
        self.assertEqual(
            response["location"],
            reverse("basket:summary"),
            "Adding item to basket should redirect to basket",
        )

        return self.client.get(reverse("checkout:index"))

    def test_payment_method(self):
        # pylint: disable=no-value-for-parameter
        response = self.setup_checkout()
        self.assertContains(response, "Dummy betaling")

    def test_payment_method_minimum(self):
        pm = PaymentMethod.objects.get()
        pm.min_amount = D(400)
        pm.save()
        # pylint: disable=no-value-for-parameter
        response = self.setup_checkout()
        self.assertNotContains(response, "Dummy betaling")

    def test_payment_method_maximum(self):
        pm = PaymentMethod.objects.get()
        pm.max_amount = D(200)
        pm.save()
        # pylint: disable=no-value-for-parameter
        response = self.setup_checkout()
        self.assertNotContains(response, "Dummy betaling")



from decimal import Decimal as D
from unittest import mock

from django.test import TestCase, override_settings
from django.urls import reverse

from oscar.core.loading import get_model

import ocyan.core.fender
from ocyan.core.utils import merge_dicts

Order = get_model("order", "Order")
Basket = get_model("basket", "Basket")

ocyan_settings = ocyan.core.fender.config.settings
CHECKOUT_OCYAN_OPTIONS = merge_dicts(
    ocyan_settings,
    {"shipping": {"enable_charged_shipping": True, "enable_free_shipping": True}},
)


class CheckoutTest(TestCase):
    fixtures = [
        "checkout/auth",
        "checkout/catalogue",
        "checkout/shipping",
        "checkout/paymentmethod",
    ]

    @mock.patch("sorl.thumbnail.templatetags.thumbnail.ThumbnailNodeBase.render")
    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUT_OCYAN_OPTIONS)
    def test_login_while_checking_out(self, render):
        "The checkout should work if I log in while checking out."
        render.return_value = ""

        self.assertEqual(Basket.objects.count(), 0, "The basket should be empty")

        basket_add_url = reverse("basket:add", args=(1,))
        body = {"quantity": 3}

        response = self.client.post(basket_add_url, body)
        self.assertEqual(
            response.status_code, 302, "Adding an item to a basket should redirect."
        )

        response = self.client.get(reverse("checkout:index"))
        self.assertEqual(
            response.status_code, 200, "The checkout page should work just fine."
        )

        post_data = {
            "csrfmiddlewaretoken": "",
            "gateway_form-username": "henk@example.com",
            "gateway_form-password": "henk",
            "gateway_form-options": "existing",
            "shipping_address_chooser-address_chooser": "1",
            "shipping_address_form-title": "",
            "shipping_address_form-first_name": "Henk",
            "shipping_address_form-last_name": "Koe",
            "shipping_address_form-line1": "Gatenkaasweg",
            "shipping_address_form-house_number": "4",
            "shipping_address_form-line2": "",
            "shipping_address_form-line3": "",
            "shipping_address_form-line4": "Ergens in Nederlan",
            "shipping_address_form-state": "",
            "shipping_address_form-postcode": "1256AB",
            "shipping_address_form-country": "NL",
            "shipping_address_form-phone_number": "",
            "billing_address_chooser-address_chooser": "5",
            "billing_address_form-email": "viggo@mandelblog.com",
            "billing_address_form-title": "",
            "billing_address_form-first_name": "Viggo",
            "billing_address_form-last_name": "de Vries",
            "billing_address_form-line1": "Weteringdijk",
            "billing_address_form-house_number": "109",
            "billing_address_form-line2": "",
            "billing_address_form-line3": "",
            "billing_address_form-line4": "emst",
            "billing_address_form-state": "",
            "billing_address_form-postcode": "8166KT",
            "billing_address_form-country": "NL",
            "shipping_method_form-method_code": "postnl",
            "payment_method_form-payment_method": "dummy:payment",
            "shipping_address_form-notes": "",
            "accepttermsofservice": "on",
        }

        response = self.client.post(reverse("checkout:index"), post_data)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(
            Order.objects.all().count(),
            0,
            "no order should be made yet because the user just logged in",
        )
        response = self.client.get(reverse("checkout:index"), follow=True)
        self.assertContains(
            response,
            "We have merged 3 items from a previous session to your basket.",
        )

        response = self.client.post(reverse("checkout:index"), post_data)
        self.assertEqual(
            Order.objects.all().count(), 1, "now the order should be created"
        )

        order = Order.objects.first()
        self.assertEqual(order.total_incl_tax, D("305.95"))
        self.assertEqual(order.total_excl_tax, D("252.85"))

    @mock.patch("sorl.thumbnail.templatetags.thumbnail.ThumbnailNodeBase.render")
    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUT_OCYAN_OPTIONS)
    @override_settings(ALLOWED_HOSTS=["*"])
    def test_create_account_while_checking_out(self, render):
        "The checkout should work if I create an account in while checking out."
        render.return_value = ""

        self.assertEqual(Basket.objects.count(), 0, "The basket should be empty")

        basket_add_url = reverse("basket:add", args=(1,))
        body = {"quantity": 3}

        response = self.client.post(basket_add_url, body)
        self.assertEqual(
            response.status_code, 302, "Adding an item to a basket should redirect."
        )

        response = self.client.get(reverse("checkout:index"))
        self.assertEqual(
            response.status_code, 200, "The checkout page should work just fine."
        )

        post_data = {
            "csrfmiddlewaretoken": "",
            "gateway_form-username": "rutantutann@example.com",
            "gateway_form-password": "henk_1234",
            "gateway_form-password2": "henk_1234",
            "gateway_form-options": "new",
            "shipping_address_chooser-address_chooser": "1",
            "shipping_address_form-title": "",
            "shipping_address_form-first_name": "Henk",
            "shipping_address_form-last_name": "Koe",
            "shipping_address_form-line1": "Gatenkaasweg",
            "shipping_address_form-house_number": "4",
            "shipping_address_form-line2": "",
            "shipping_address_form-line3": "",
            "shipping_address_form-line4": "Ergens in Nederlan",
            "shipping_address_form-state": "",
            "shipping_address_form-postcode": "1256AB",
            "shipping_address_form-country": "NL",
            "shipping_address_form-phone_number": "",
            "billing_address_chooser-address_chooser": "5",
            "billing_address_form-email": "viggo@mandelblog.com",
            "billing_address_form-title": "",
            "billing_address_form-first_name": "Viggo",
            "billing_address_form-last_name": "de Vries",
            "billing_address_form-line1": "Weteringdijk",
            "billing_address_form-house_number": "109",
            "billing_address_form-line2": "",
            "billing_address_form-line3": "",
            "billing_address_form-line4": "emst",
            "billing_address_form-state": "",
            "billing_address_form-postcode": "8166KT",
            "billing_address_form-country": "NL",
            "shipping_method_form-method_code": "postnl",
            "payment_method_form-payment_method": "dummy:payment",
            "shipping_address_form-notes": "",
            "accepttermsofservice": "on",
        }

        response = self.client.post(reverse("checkout:index"), post_data)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(Order.objects.all().count(), 1, "The order should be created")

        order = Order.objects.first()
        self.assertEqual(order.total_incl_tax, D("305.95"))
        self.assertEqual(order.total_excl_tax, D("252.85"))



# -*- coding: utf-8 -*-

__all__ = ["OcyanOscar3CheckoutPluginConfig"]


def __getattr__(name):
    if name == "OcyanOscar3CheckoutPluginConfig":
        from .entrypoint import OcyanOscar3CheckoutPluginConfig

        return OcyanOscar3CheckoutPluginConfig
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")



import logging

from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.db.models.signals import post_save
from django.dispatch import receiver

from oscar.core.loading import get_model

from ocyan.plugin.oscar_checkout.plugins import get_payment_method_choices

logger = logging.getLogger(__name__)

PaymentMethod = get_model("checkout", "PaymentMethod")
Country = get_model("address", "Country")


def setup_payment_methods(sender, app_config, **kwargs):
    try:
        methods = get_payment_method_choices()

        for key, label in methods:
            try:
                payment_method, created = PaymentMethod.objects.get_or_create(
                    key=key, defaults={"label": label}
                )
                if created:
                    logger.info("Created payment_method key=%s label=%s.", key, label)
                    payment_method.country.set(Country.objects.all())
            except PaymentMethod.MultipleObjectsReturned:
                pass  # multiple is at least one so nomatter.
    except Exception as e:  # pylint: disable=W0703
        logger.exception(e)


@receiver(post_save, sender=PaymentMethod)
def handle_payment_method_change(sender, instance, raw, **kwargs):
    if not raw:  # don't run this hook when loading a fixture
        if instance.requires_permission:
            ctype = ContentType.objects.get_for_model(sender)
            Permission.objects.get_or_create(
                codename=instance.key,
                content_type=ctype,
                defaults={"name": "Allow %s" % instance.label},
            )



from django.conf import settings
from django.utils.translation import gettext_lazy as _

from oscar.apps.payment.forms import BillingAddressForm as BaseBillingAddressForm


class BillingAddressForm(BaseBillingAddressForm):
    phone_number_fields = {}

    field_order = settings.OSCAR_BILLING_FORM_FIELD_ORDER

    class Meta(BaseBillingAddressForm.Meta):
        fields = settings.OSCAR_BILLING_FORM_FIELDS
        help_texts = {"email": _("The invoice will be sent to this e-mail address")}



# Generated by Django 2.2.9 on 2020-04-22 11:49

from django.db import migrations, models

from ocyan.core.fender import config

class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0005_payment_method_limits'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymentmethod',
            name='send_invoice',
            field=models.BooleanField(default=config.get("oscar_communications", "invoice", False)),
        ),
    ]



# Generated by Django 3.2.13 on 2022-06-07 11:23

from django.db import migrations
from django.db.models import Q

from ocyan.core.fender import config


def forwards(apps, schema_editor):
    FixedSurcharge = apps.get_model("checkout", "FixedSurcharge")
    USE_PRICE_INCL_TAX = config.get("oscar", "use_price_incl_tax", True)
    
    for surcharge in FixedSurcharge.objects.all():
        if USE_PRICE_INCL_TAX:
            surcharge.price = surcharge.incl_tax
        else:
            surcharge.price = surcharge.excl_tax
            
        surcharge.save()



class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0015_add_vat_rates_surcharges'),
    ]

    operations = [
        migrations.RunPython(forwards, migrations.RunPython.noop),
    ]



# Generated by Django 3.2.13 on 2022-06-07 11:21

from django.db import migrations, models
import oscar.models.fields.autoslugfield


class Migration(migrations.Migration):

    dependencies = [
        ('address', '0001_initial'),
        ('checkout', '0011_auto_20210818_1542'),
        ('address', '0008_useraddress_email'),
    ]

    operations = [
        migrations.CreateModel(
            name='FixedSurcharge',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=32)),
                ('code', oscar.models.fields.autoslugfield.AutoSlugField(blank=True, editable=False, max_length=128, populate_from='name', unique=True, verbose_name='Code')),
                ('excl_tax', models.DecimalField(decimal_places=2, max_digits=12)),
                ('incl_tax', models.DecimalField(decimal_places=2, max_digits=12)),
                ('country', models.ManyToManyField(blank=True, to='address.Country', verbose_name='Countries')),
                ('payment_method', models.ManyToManyField(blank=True, to='checkout.PaymentMethod', verbose_name='Payment methods')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PercentageSurcharge',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=32)),
                ('code', oscar.models.fields.autoslugfield.AutoSlugField(blank=True, editable=False, max_length=128, populate_from='name', unique=True, verbose_name='Code')),
                ('percentage', models.DecimalField(decimal_places=2, max_digits=12)),
                ('include_shipping_charges', models.BooleanField(default=True)),
                ('country', models.ManyToManyField(blank=True, to='address.Country', verbose_name='Countries')),
                ('payment_method', models.ManyToManyField(blank=True, to='checkout.PaymentMethod', verbose_name='Payment methods')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.forms import fields, widgets


class StandardsCompliantRadioWidget(widgets.RadioSelect):
    option_template_name = "checkout/widgets/radio_options.html"


class MethodsChoiceField(fields.ChoiceField):
    def __init__(self, *, choices=(), methods=(), **kwargs):
        super().__init__(**kwargs)
        self.choices = choices
        self.methods = methods



# pylint: disable=W0622
import collections
import logging

from django.template.loader import render_to_string
from django.utils.translation import gettext as _
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.renderers import StaticHTMLRenderer
from rest_framework.response import Response
from rest_framework.reverse import reverse
from rest_framework.settings import api_settings
from rest_framework.views import APIView

from oscar.apps.address.models import Country
from oscar.core.loading import get_class, get_model

from oscarapi.views.basket import ShippingMethodView as BaseShippingMethodView

from ocyan.main.loading import get_ocyan_class

from ocyan.plugin.oscar_checkout.checkout.forms import (
    PaymentMethodForm,
    ShippingMethodForm,
)
from ocyan.plugin.oscar_checkout.checkout.utils import get_default_shipping_method
from ocyan.plugin.oscar_checkout.checkout.views import IndexView
from ocyan.plugin.oscar_checkout.utils import country_from_url

Repository = get_class("shipping.repository", "Repository")
ShippingAddress = get_model("order", "ShippingAddress")

payment_method_choices = get_class("checkout.utils", "payment_method_choices")

from .serializers import (
    OrderTotalSerializer,
    PaymentMethodSerializer,
    ShippingCountrySerializer,
    ShippingMethodSerializer,
)

CheckoutSerializer = get_ocyan_class("checkoutapi.serializers", "CheckoutSerializer")

logger = logging.getLogger(__name__)


@api_view(("GET",))
def api_root(request, format=None):
    apis = [
        (
            "login",
            reverse(
                "api-login", request=request, format=format, current_app="checkoutapi"
            ),
        ),
        (
            "basket-shipping-methods",
            reverse(
                "api-basket-shipping-methods",
                request=request,
                format=format,
                current_app="checkoutapi",
            ),
        ),
        (
            "basket-total",
            reverse(
                "api-basket-total",
                request=request,
                format=format,
                current_app="checkoutapi",
            ),
        ),
        (
            "countries",
            reverse(
                "country-list",
                request=request,
                format=format,
                current_app="checkoutapi",
            ),
        ),
    ]
    return Response(collections.OrderedDict(apis))


class ShippingMethodView(BaseShippingMethodView):
    serializer_class = ShippingCountrySerializer
    shipping_method_serializer_class = ShippingMethodSerializer
    renderer_classes = [StaticHTMLRenderer] + api_settings.DEFAULT_RENDERER_CLASSES

    def render_snippet(self, request, shiping_methods, basket, shipping_address=None):
        default_shipping_method = get_default_shipping_method(request, shipping_address)
        context = {
            "nolabel": True,
            "form": ShippingMethodForm(
                prefix="shipping_method_form",
                methods=shiping_methods,
                default=(
                    default_shipping_method.code
                    if default_shipping_method is not None
                    else None
                ),
            ),
            "title": _("Shipping method"),
            "request": request,
            "id": "shipping_method_form",
            "basket": basket,
        }

        return render_to_string("checkout/partials/shipping_method_block.html", context)

    def _get(self, request, shipping_address=None, format=None):  # pylint: disable=redefined-builtin
        basket = request.basket

        if shipping_address is not None:
            request.META["shipping_country"] = shipping_address.country.pk

        shiping_methods = Repository().get_shipping_methods(
            basket=basket,
            user=request.user,
            shipping_addr=shipping_address,
            request=request,
        )
        if format == "html":
            return Response(
                self.render_snippet(
                    request, shiping_methods, basket, shipping_address=shipping_address
                )
            )

        ser = self.shipping_method_serializer_class(
            shiping_methods, many=True, context={"basket": basket}
        )
        return Response(ser.data)

    def post(self, request, *args, format=None, **kwargs):  # pylint: disable=redefined-builtin
        s_ser = self.serializer_class(
            data=request.data, context={"request": request}, partial=True
        )
        shipping_address = None
        if s_ser.is_valid():
            shipping_address = ShippingAddress(**s_ser.validated_data)

        return self._get(request, format=format, shipping_address=shipping_address)


class OrderTotalView(APIView):
    serializer_class = CheckoutSerializer

    def get(self, request, format=None):
        return Response(
            {
                "shipping_method_code": "no-shipping-required",
                "shipping_address": {
                    "country": "http://127.0.0.1:9000/api/countries/nl/",
                    "first_name": "Henk",
                    "last_name": "Van den Heuvel",
                    "line1": "Roemerlaan 44",
                    "line2": "",
                    "line3": "",
                    "line4": "Kroekingen",
                    "notes": "Niet STUK MAKEN OK!!!!",
                    "phone_number": "+31 26 370 4887",
                    "postcode": "7777KK",
                    "state": "Gerendrecht",
                    "title": "Mr",
                },
            }
        )

    def post(self, request, format=None):
        data = request.data

        # this code is only needed for vat plugin
        try:
            shipping_country_url = data["shipping_address"]["country"]
            shipping_country = country_from_url(shipping_country_url)
            request.META["shipping_country"] = shipping_country
        except KeyError:
            pass

        basket = request.basket

        if basket.is_empty:
            return Response(status=status.HTTP_406_NOT_ACCEPTABLE)

        try:
            data["basket"] = reverse(
                "basket-detail",
                args=(basket.pk,),
                request=request,
                format=format,
                current_app="checkoutapi",
            )

        except TypeError:  # if data is a string; TypeError occurs
            return Response(status=status.HTTP_406_NOT_ACCEPTABLE)

        ser = self.serializer_class(
            data=data, context={"request": request}, partial=True
        )
        if ser.is_valid():
            checkout_data = ser.validated_data
            context = {
                "nohide": True,
                "id": "checkouttotal",
                "title": _("Your order"),
                "request": request,
                "user": request.user,
                "order_total": checkout_data["total"],
            }
            context.update(checkout_data)
            snippet = render_to_string("checkout/partials/totals_block.html", context)

            result_data = {"snippet": snippet}
            result_data.update(context)
            return Response(OrderTotalSerializer(result_data).data)

        return Response(ser.errors, status.HTTP_406_NOT_ACCEPTABLE)


class PaymentMethodView(APIView):
    serializer_class = PaymentMethodSerializer

    def get(self, request, format=None):
        return Response(
            IndexView(
                request=self.request, args=self.args, kwargs=self.kwargs
            ).get_payment_methods()
        )

    def post(self, request, format=None):
        data = request.data
        ser = self.serializer_class(data=data, context={"request": request})
        if ser.is_valid():
            val_data = ser.validated_data
            try:
                country = Country.objects.get(iso_3166_1_a2=val_data.get("country"))
            except Country.DoesNotExist:
                country = Country.objects.filter(is_shipping_country=True).first()

            results = payment_method_choices(
                request.user, request.basket, country=country
            )

            context = {
                "nolabel": True,
                "form": PaymentMethodForm(
                    prefix="payment_method_form", payment_method_choices=results
                ),
                "title": _("Payment method"),
                "request": request,
                "id": "payment_method_form",
            }

            snippet = render_to_string(
                "checkout/partials/payment_method_block.html", context
            )

            result_data = {"snippet": snippet}
            result_data.update(context)

            return Response(PaymentMethodSerializer(result_data).data)

        return Response(ser.errors, status.HTTP_406_NOT_ACCEPTABLE)



from django import template

from oscar.core.loading import get_class

DeliveryTime = get_class("checkout.delivery", "DeliveryTime")
register = template.Library()


@register.simple_tag(takes_context=True)
def delivery_time(context, basket):
    return DeliveryTime.delivery_time(basket, context.get("request"))



# -*- coding: utf-8 -*-
from django.conf.urls.i18n import i18n_patterns
from django.urls import path

from ocyan.core.fender import config
from ocyan.core.utils import merge_dicts

from ocyan.plugin.oscar_checkout.settings import (
    OCYAN_CHECKOUT_OSCAR_SETTINGS,
    OCYAN_REST_FRAMEWORK_SETTINGS,
)


class CheckoutEntryPoint:
    def contribute_to_settings(self, settings):
        settings = merge_dicts(
            settings, OCYAN_REST_FRAMEWORK_SETTINGS, deduplicate_iterables=True
        )
        return merge_dicts(settings, OCYAN_CHECKOUT_OSCAR_SETTINGS, overwrite=True)

    def contribute_to_middleware(self, middleware):
        middlewares = middleware + ["oscarapi.middleware.ApiBasketMiddleWare"]
        try:
            middlewares.remove("oscar.apps.basket.middleware.BasketMiddleware")
        except ValueError:
            pass
        return middlewares

    def get_api_application(self):
        raise NotImplementedError

    def contribute_to_urlpatterns(self, urlpatterns):
        api = self.get_api_application()

        if config.i18n_enabled:
            urls = i18n_patterns(
                path(r"checkout/api/", api.urls), prefix_default_language=False
            )
        else:
            urls = [path(r"checkout/api/", api.urls)]
        urls.extend(list(urlpatterns))
        return urls

    def get_smoketest_modules(self):
        return ["ocyan.plugin.oscar_checkout.tests.test_smoketests"]

    def contribute_to_i18n_fields(self, fields_dict):
        fields_dict.update(
            {
                "checkout.paymentmethod.label": True,
                "checkout.percentagesurcharge.name": True,
                "checkout.fixedsurcharge.name": True,
            }
        )
        return fields_dict



from decimal import Decimal
from unittest import mock

from django.conf import settings
from django.contrib.auth import get_user_model
from django.test import TestCase
from django.urls import reverse
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_encode
from django.utils.translation import gettext

from oscar.core.loading import get_model

import ocyan.core.fender
from ocyan.core.utils import merge_dicts
from ocyan.main.utils import randomtokengenerator
from ocyan.plugin.oscar.pipeline import PIPELINE_STATUS_NEW

from ..checkout.models import FixedSurcharge

User = get_user_model()
Order = get_model("order", "Order")
Surcharge = get_model("order", "Surcharge")
Country = get_model("address", "Country")
PaymentMethod = get_model("checkout", "PaymentMethod")
FixedSurcharge = get_model("checkout", "FixedSurcharge")
PercentageSurcharge = get_model("checkout", "PercentageSurcharge")

ocyan_settings = ocyan.core.fender.config.settings
CHECKOUT_OCYAN_OPTIONS = merge_dicts(
    ocyan_settings,
    {"shipping": {"enable_charged_shipping": True, "enable_free_shipping": True}},
)


class CheckoutTryAgainTest(TestCase):
    fixtures = [
        "checkout/auth",
        "checkout/catalogue",
        "checkout/shipping",
        "checkout/paymentmethod",
        "checkout/basket",
        "checkout/order",
    ]

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUT_OCYAN_OPTIONS)
    def test_no_problems_just_an_empty_basket(self):
        "No errors have occurred during ordering the user just has an empty basket"
        if not settings.OSCAR_ALLOW_ANON_CHECKOUT:
            henk = User.objects.get(username="henk")
            self.client.force_login(henk)

        self.client.get(reverse("basket:summary"))

        response = self.client.get(reverse("checkout:index"), follow=True)
        self.assertTemplateNotUsed(
            response,
            "checkout/try_again.html",
            msg_prefix="no redirect should have happened to the try again view",
        )
        self.assertTemplateUsed(
            response,
            "checkout/onestep.html",
            msg_prefix="The onestep checkout template should have been rendered",
        )
        self.assertContains(
            response,
            "<h1>%s</h1>" % gettext("Your shopping basket appears to be empty"),
            msg_prefix="Standard message should be displayed",
        )

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUT_OCYAN_OPTIONS)
    def test_error_checkout_visited_with_unfinished_order_in_session(self):
        "When a payment fails, the user should be redirected to the try again page"
        if not settings.OSCAR_ALLOW_ANON_CHECKOUT:
            henk = User.objects.get(username="henk")
            self.client.force_login(henk)

        self.client.get(reverse("basket:summary"))

        s = self.client.session
        s["checkout_order_id"] = 1
        s.save()

        response = self.client.get(reverse("checkout:index"), follow=True)
        self.assertTemplateUsed(
            response,
            "checkout/try_again.html",
            msg_prefix="we should be redirected to the try again page",
        )
        self.assertTemplateNotUsed(
            response,
            "checkout/onestep.html",
            msg_prefix="the onstep html should not be rendered",
        )
        self.assertNotContains(
            response,
            "<h1>%s</h1>" % gettext("Your shopping basket appears to be empty"),
            msg_prefix="No standard error message should be displayed",
        )
        self.assertRedirects(
            response,
            reverse("checkout:try-again"),
            status_code=302,
            target_status_code=200,
            fetch_redirect_response=True,
        )

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUT_OCYAN_OPTIONS)
    def test_error_checkout_visited_with_bugged_order_in_session(self):
        "When a connection error happend during checkout the user should be lead to the try again page"
        o = Order.objects.get()
        o.status = PIPELINE_STATUS_NEW
        o.save()

        if not settings.OSCAR_ALLOW_ANON_CHECKOUT:
            henk = User.objects.get(username="henk")
            self.client.force_login(henk)

        self.client.get(reverse("basket:summary"))

        s = self.client.session
        s["checkout_order_id"] = 1
        s.save()

        response = self.client.get(reverse("checkout:index"), follow=True)
        self.assertTemplateUsed(
            response,
            "checkout/try_again.html",
            msg_prefix="we should be redirected to the try again page",
        )
        self.assertTemplateNotUsed(
            response,
            "checkout/onestep.html",
            msg_prefix="the onstep html should not be rendered",
        )
        self.assertNotContains(
            response,
            "<h1>%s</h1>" % gettext("Your shopping basket appears to be empty"),
            msg_prefix="No standard error message should be displayed",
        )
        self.assertRedirects(
            response,
            reverse("checkout:try-again"),
            status_code=302,
            target_status_code=200,
            fetch_redirect_response=True,
        )

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUT_OCYAN_OPTIONS)
    def test_error_checkout_post_with_unfinished_order_in_session(self):
        "When a payment fails, the user should be redirected to the try again page"
        if not settings.OSCAR_ALLOW_ANON_CHECKOUT:
            henk = User.objects.get(username="henk")
            self.client.force_login(henk)

        self.client.get(reverse("basket:summary"))

        s = self.client.session
        s["checkout_order_id"] = 1
        s.save()

        response = self.client.post(
            reverse("checkout:index"),
            {"payment_method_form-payment_method": "dummy:payment"},
            follow=True,
        )
        self.assertTemplateUsed(
            response,
            "checkout/try_again.html",
            msg_prefix="we should be redirected to the try again page",
        )
        self.assertTemplateNotUsed(
            response,
            "checkout/onestep.html",
            msg_prefix="the onstep html should not be rendered",
        )

        self.assertNotContains(
            response,
            "<h1>%s</h1>" % gettext("Your shopping basket appears to be empty"),
            msg_prefix="No standard error message should be displayed",
        )
        self.assertRedirects(
            response,
            reverse("checkout:try-again"),
            status_code=302,
            target_status_code=200,
            fetch_redirect_response=True,
        )

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUT_OCYAN_OPTIONS)
    def test_error_checkout_post_with_bugged_order_in_session(self):
        "When a connection error happend during checkout the user should be lead to the try again page"
        o = Order.objects.get()
        o.status = PIPELINE_STATUS_NEW
        o.save()

        if not settings.OSCAR_ALLOW_ANON_CHECKOUT:
            henk = User.objects.get(username="henk")
            self.client.force_login(henk)

        self.client.get(reverse("basket:summary"))

        s = self.client.session
        s["checkout_order_id"] = 1
        s.save()

        # as long as the payment method is post everything should be fine
        response = self.client.post(
            reverse("checkout:index"),
            {"payment_method_form-payment_method": "dummy:payment"},
            follow=True,
        )

        self.assertTemplateUsed(
            response,
            "payment/dummy/payment.html",
            msg_prefix="we should be redirected to the payment page",
        )
        self.assertTemplateNotUsed(
            response,
            "checkout/try_again.html",
            msg_prefix="we should not be redirected to the try again page",
        )
        self.assertTemplateNotUsed(
            response,
            "checkout/onestep.html",
            msg_prefix="the onstep html should not be rendered",
        )
        self.assertNotContains(
            response,
            "<h1>%s</h1>" % gettext("Your shopping basket appears to be empty"),
            msg_prefix="No standard error message should be displayed",
        )
        self.assertRedirects(
            response,
            reverse("dummy:payment"),
            status_code=302,
            target_status_code=200,
            fetch_redirect_response=True,
        )

    def test_tryagain_from_link(self):
        "try again from link should work"
        # make henk logged in
        henk = User.objects.get(username="henk")
        self.client.force_login(henk)

        order = Order.objects.get(pk=1)
        order.user = henk
        order.save()
        token = randomtokengenerator.make_token(order.user)
        base64_order_id = urlsafe_base64_encode(force_bytes(order.pk))

        url = reverse(
            "checkout:try-again-from-link",
            kwargs={"oidb64": base64_order_id, "token": token},
        )
        self.assertTrue(randomtokengenerator.check_token(henk, token))

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_hack_tryagain_from_link(self):
        "trying again from link of somebody else does not work"
        # make henk logged in
        henk = User.objects.get(username="henk")
        self.client.force_login(henk)

        order = Order.objects.get(pk=1)
        order.user = henk
        order.save()
        token = randomtokengenerator.make_token(order.user)
        base64_order_id = urlsafe_base64_encode(force_bytes(order.pk))

        url = reverse(
            "checkout:try-again-from-link",
            kwargs={"oidb64": base64_order_id, "token": token},
        )
        self.assertTrue(randomtokengenerator.check_token(henk, token))

        henk = User.objects.get(username="admin")
        self.client.force_login(henk)

        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_try_again_anonymous(self):
        "Anonymous users should be able to pay their open order"
        order = Order.objects.get(pk=1)
        token = order.verification_hash()
        base64_order_id = urlsafe_base64_encode(force_bytes(order.pk))

        url = reverse(
            "checkout:try-again-from-link",
            kwargs={"oidb64": base64_order_id, "token": token},
        )
        self.assertTrue(order.check_verification_hash(token))

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_hack_again_anonymous(self):
        "Anonymous hackers should not be able to pay their open order"
        order = Order.objects.get(pk=1)
        token = "5ul-cc8bf994d7002701c70e"
        base64_order_id = urlsafe_base64_encode(force_bytes(order.pk))

        url = reverse(
            "checkout:try-again-from-link",
            kwargs={"oidb64": base64_order_id, "token": token},
        )
        self.assertFalse(order.check_verification_hash(token))

        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_try_again_on_order_with_surcharges(self):
        """
        When an order has surcharges, the try again should on initial page load re-calculate
        these surcharges. As the customer is able to choose a different payment method, scenarios
        can have changed since the order was created (eg new surcharges added, or surcharges removed).

        The try again does the following things;
        - re-calculate the surcharges
        - re-calculate the order total
        - upon POST, the actual totals will also be saved (initially (GET) it's just displayed).

        """
        order = Order.objects.get(pk=1)
        token = order.verification_hash()
        base64_order_id = urlsafe_base64_encode(force_bytes(order.pk))

        # add a surcharge to the order
        order_surcharge = order.surcharges.create(
            name="test",
            code="test",
            incl_tax=Decimal("10.00"),
            excl_tax=Decimal("8.26"),
        )
        order.total_incl_tax += order_surcharge.incl_tax
        order.total_excl_tax += order_surcharge.excl_tax
        order.save()

        # Ensure the order totals are as expected (with surcharge included)
        self.assertEqual(order.total_incl_tax, Decimal("141.95"))
        self.assertEqual(order.total_excl_tax, Decimal("117.31"))

        url = reverse(
            "checkout:try-again-from-link",
            kwargs={"oidb64": base64_order_id, "token": token},
        )

        # Surcharges should be empty on initial page load.
        response = self.client.get(url)
        self.assertEqual(response.context["surcharges"], None)

        # Assert order totals from response.context["order"], this should not include the surcharge
        self.assertEqual(response.context["order"].total_incl_tax, Decimal("131.95"))
        self.assertEqual(response.context["order"].total_excl_tax, Decimal("109.05"))

        # Now POST the form, there are no surcharges yet for dummy payment, so the order total should be the same
        response = self.client.post(
            url,
            {"_try-again": "try-again", "payment_method": "dummy:payment"},
            follow=True,
        )
        order.refresh_from_db()
        self.assertEqual(order.total_incl_tax, Decimal("131.95"))
        self.assertEqual(order.total_excl_tax, Decimal("109.05"))

        # Create a fixed surcharge for the dummy payment method and try again.
        fixed_charge = FixedSurcharge.objects.create(
            name="Dummy payment surcharge",
            code="dummy-payment-surcharge",
            price=Decimal("500.00"),
        )
        fixed_charge.payment_method.set(
            PaymentMethod.objects.filter(key="dummy:payment")
        )
        fixed_charge.country.set(Country.objects.all())

        # Now POST the form, there are surcharges for dummy payment, so the order total should be updated
        response = self.client.post(
            url,
            {"_try-again": "try-again", "payment_method": "dummy:payment"},
            follow=True,
        )
        order.refresh_from_db()
        self.assertEqual(order.total_incl_tax, Decimal("631.95"))
        self.assertEqual(order.total_excl_tax, Decimal("522.27"))

        # Create a second surcharge for the dummy payment method and try again.
        percentage_surcharge = PercentageSurcharge.objects.create(
            name="Dummy payment percentage surcharge",
            code="dummy-payment-percentage-surcharge",
            percentage=Decimal("10.00"),
            include_shipping_charges=True,
        )
        percentage_surcharge.payment_method.set(
            PaymentMethod.objects.filter(key="dummy:payment")
        )
        percentage_surcharge.country.set(Country.objects.all())

        # Now POST the form, there are multiple surcharges for dummy payment, so the order total should be updated
        response = self.client.post(
            url,
            {"_try-again": "try-again", "payment_method": "dummy:payment"},
            follow=True,
        )
        order.refresh_from_db()
        # We created a percentage charge that INCLDUES shipping charges.
        # This means the percentage is calculated over: basket.total_incl_tax + shipping_charge.incl_tax
        # The shipping charge is 6.95 (incl), so the percentage is calculated over 125.00 + 6.95 = 131.95
        # The percentage is 10%, so the surcharge is 13.195 (rounded to 13.20)
        # The total should be 125.00 + 6.95 + 500 (fixed surcharge) + 13.20 (percentage surcharge) = 645.15
        self.assertEqual(order.total_incl_tax, Decimal("645.15"))
        self.assertEqual(order.total_excl_tax, Decimal("533.18"))

    def test_try_again_render_totals(self):
        """Test the render_totals handler for Javascript request"""
        order = Order.objects.get(pk=1)
        token = order.verification_hash()
        base64_order_id = urlsafe_base64_encode(force_bytes(order.pk))

        url = reverse(
            "checkout:try-again-from-link",
            kwargs={"oidb64": base64_order_id, "token": token},
        )

        # Simulate AJAX request
        response = self.client.get(
            url + "?payment_method=dummy:payment",
            HTTP_X_REQUESTED_WITH="XMLHttpRequest",
        )
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "checkout/partials/try_again_totals_block.html"
        )
        self.assertTemplateNotUsed(response, "checkout/try_again.html")

        self.assertEqual(response.context["order"].total_incl_tax, Decimal("131.95"))
        self.assertEqual(response.context["order"].total_excl_tax, Decimal("109.05"))

        # Create a fixed surcharge for the dummy payment method and try again.
        fixed_charge = FixedSurcharge.objects.create(
            name="Dummy payment surcharge",
            code="dummy-payment-surcharge",
            price=Decimal("500.00"),
        )
        fixed_charge.payment_method.set(
            PaymentMethod.objects.filter(key="dummy:payment")
        )
        fixed_charge.country.set(Country.objects.all())

        # Simulate AJAX request
        response = self.client.get(
            url + "?payment_method=dummy:payment",
            HTTP_X_REQUESTED_WITH="XMLHttpRequest",
        )
        self.assertEqual(response.context["order"].total_incl_tax, Decimal("631.95"))
        self.assertEqual(response.context["order"].total_excl_tax, Decimal("522.2731"))

        # Create a second surcharge for the dummy payment method and try again.
        percentage_surcharge = PercentageSurcharge.objects.create(
            name="Dummy payment percentage surcharge",
            code="dummy-payment-percentage-surcharge",
            percentage=Decimal("10.00"),
            include_shipping_charges=False,
        )
        percentage_surcharge.payment_method.set(
            PaymentMethod.objects.filter(key="dummy:payment")
        )
        percentage_surcharge.country.set(Country.objects.all())

        # Now POST the form, there are multiple surcharges for dummy payment, so the order total should be updated
        response = self.client.get(
            url + "?payment_method=dummy:payment",
            HTTP_X_REQUESTED_WITH="XMLHttpRequest",
        )
        # We created a percentage charge that EXCLUDES shipping charges.
        # This means the percentage is calculated over: basket.total_incl_tax only
        # The base amount is 125.00
        # The percentage is 10%, so the surcharge is 12.50
        # The shipping charge is 6.95 (incl)
        # The total should be 125.00 + 6.95 + 500 (fixed surcharge) + 12.50 (percentage surcharge) = 644.45
        self.assertEqual(response.context["order"].total_incl_tax, Decimal("644.45"))
        self.assertEqual(response.context["order"].total_excl_tax, Decimal("532.6037"))



# Generated by Django 3.2.19 on 2023-07-10 10:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0016_surcharge_flat_rate_price_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='fixedsurcharge',
            name='excl_tax',
        ),
        migrations.RemoveField(
            model_name='fixedsurcharge',
            name='incl_tax',
        ),
    ]



from django.conf import settings
from django.urls import reverse


class IsApiRequest:
    @staticmethod
    def is_api_request(request):
        path = request.path.lower()
        for root_url in settings.OSCARAPI_ROOT_URLS:
            api_root = reverse(root_url).lower()
            if path.startswith(api_root):
                return True

        return False



# -*- coding: utf-8 -*-
from django.apps import apps

from ocyan.core.plugins import OcyanPluginConfig

from .shared.entrypoint import CheckoutEntryPoint


class OcyanOscar3CheckoutPluginConfig(CheckoutEntryPoint, OcyanPluginConfig):
    def contribute_to_installed_apps(self, installed_apps):
        installed_apps.apps.extend(
            (
                "rest_framework",
                "ocyan.plugin.oscar_checkout.checkoutapi.apps.Oscar3ApiAppConfig",
            )
        )
        installed_apps.oscar.append(
            "ocyan.plugin.oscar_checkout.checkout.apps.CheckoutConfig",
            replaces="oscar.apps.checkout.apps.CheckoutConfig",
        )
        installed_apps.oscar.register_extension(
            "ocyan.plugin.oscar_checkout.oxyan_checkout_extension",
            "ocyan.plugin.oscar_checkout.checkout",
        )
        installed_apps.oscar.register_extension(
            "ocyan.plugin.oscar_checkout.payment_co_extension", "oscar.apps.payment"
        )
        installed_apps.django.append("ocyan.plugin.oscar_checkout")
        return installed_apps

    def get_api_application(self):
        return apps.get_app_config("checkoutapi")



from django.db.models.signals import post_migrate
from django.utils.translation import gettext_lazy as _

# pylint: disable=no-name-in-module
from oscar.core.application import OscarConfig
from oscar.core.loading import get_class

from ocyan.plugin.oscar_checkout.shared.appconfig import CheckoutAppConfig


class CheckoutConfig(CheckoutAppConfig, OscarConfig):
    label = "checkout"
    name = "ocyan.plugin.oscar_checkout.checkout"
    namespace = "checkout"
    verbose_name = _("Checkout")

    def ready(self):
        self.index_view = get_class("checkout.views", "IndexView")
        self.cancel_view = get_class("checkout.views", "CancelView")
        self.thankyou_view = get_class("checkout.views", "ThankYouView")
        self.thankyou_from_link_view = get_class(
            "checkout.views", "ThankYouFromLinkView"
        )
        self.order_not_found_view = get_class("checkout.views", "OrderNotFoundView")
        self.try_again_view = get_class("checkout.views", "TryAgainView")
        self.try_again_from_link_view = get_class(
            "checkout.views", "TryAgainFromLinkView"
        )

        from ocyan.plugin.oscar_checkout.checkout.signal_handlers import (
            setup_payment_methods,
        )

        post_migrate.connect(setup_payment_methods, sender=self)



from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.urls import path


class CheckoutAppConfig:
    index_view = None
    cancel_view = None
    thankyou_view = None
    thankyou_from_link_view = None
    order_not_found_view = None
    try_again_view = None
    try_again_from_link_view = None

    def get_urls(self):
        # pylint: disable=no-member
        urls = [
            path("", self.index_view.as_view(), name="index"),
            path("cancel/", self.cancel_view.as_view(), name="cancel"),
            path("thank-you/", self.thankyou_view.as_view(), name="thank-you"),
            path(
                "thank-you/<oidb64>/<token>/",
                self.thankyou_from_link_view.as_view(),
                name="thank-you",
            ),
            path(
                "order-not-found/",
                self.order_not_found_view.as_view(),
                name="order-not-found",
            ),
            path("try-again/", self.try_again_view.as_view(), name="try-again"),
            path(
                "try-again/<oidb64>/<token>/",
                self.try_again_from_link_view.as_view(),
                name="try-again-from-link",
            ),
        ]
        return self.post_process_urls(urls)

    def get_url_decorator(self, pattern):
        if not settings.OSCAR_ALLOW_ANON_CHECKOUT:
            return login_required
        if pattern.name.startswith("user-address"):
            return login_required
        return None



from django.db import models
from django.utils.translation import gettext

from oscar.core.loading import get_class, get_model

Repository = get_class("shipping.repository", "Repository")

PaymentMethod = get_model("checkout", "PaymentMethod")


def payment_method_choices(user, basket, **filters):
    total = basket.total_incl_tax

    return [
        (method.key, gettext(method.label))
        for method in PaymentMethod.objects.filter(
            models.Q(max_amount__isnull=True) | models.Q(max_amount__gte=total),
            min_amount__lte=total,
            **filters,
        )
        if method.has_permission(user)
    ]


def get_available_shipping_methods(request, shipping_address=None):
    """
    Returns all applicable shipping method objects for a given basket.
    """
    # Shipping methods can depend on the user, the contents of the basket
    # and the shipping address (so we pass all these things to the
    # repository).  I haven't come across a scenario that doesn't fit this
    # system.
    return Repository().get_shipping_methods(
        basket=request.basket,
        user=request.user,
        shipping_addr=shipping_address,
        request=request,
    )


def get_default_shipping_method(request, shipping_address=None):
    return Repository().get_default_shipping_method(
        basket=request.basket,
        user=request.user,
        shipping_addr=shipping_address,
        request=request,
    )


def clean_shipping_method(request, basket, shipping_method_code, shipping_address):
    repo = Repository()

    default = repo.get_default_shipping_method(
        basket=basket,
        user=request.user,
        request=request,
        shipping_addr=shipping_address,
    )

    if shipping_method_code is not None:
        methods = repo.get_shipping_methods(
            basket=basket,
            user=request.user,
            request=request,
            shipping_addr=shipping_address,
        )

        find_method = (s for s in methods if s.code == shipping_method_code)
        shipping_method = next(find_method, default)
        return shipping_method

    return default



from decimal import Decimal as D
from unittest import mock

from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.test import TestCase
from django.urls import reverse
from phonenumber_field.phonenumber import PhoneNumber
from rest_framework.reverse import reverse as api_reverse
from rest_framework.settings import api_settings
from rest_framework.test import APIRequestFactory

from oscar.apps.checkout.applicator import SurchargeApplicator
from oscar.core.loading import get_model

import ocyan.core.fender
from ocyan.plugin.wagtail_utils.utils import get_wagtailsite

from ocyan.plugin.oscar_checkout.checkoutapi import serializers

Basket = get_model("basket", "Basket")
Country = get_model("address", "Country")
PaymentMethod = get_model("checkout", "PaymentMethod")
ShippingMethod = get_model("shipping", "OrderAndItemCharges")
FixedSurcharge = get_model("checkout", "FixedSurcharge")
PercentageSurcharge = get_model("checkout", "PercentageSurcharge")
User = get_user_model()
factory = APIRequestFactory()


CHECKOUTAPI_OCYAN_OPTIONS = {"shipping": {"enable_charged_shipping": True}}


class TestSurchargeApplicator(SurchargeApplicator):
    def get_surcharges(self, basket, **kwargs):
        return (
            FixedSurcharge(price=D(20), vat_rate="high"),
            PercentageSurcharge(percentage=10, include_shipping_charges=True),
        )


class CheckoutApiTest(TestCase):
    fixtures = [
        "checkout/auth",
        "checkout/catalogue",
        "checkout/shipping",
        "checkout/paymentmethod",
    ]

    def fill_basket(self):
        henk = User.objects.get(username="henk")
        self.client.force_login(henk)
        self.assertEqual(Basket.objects.count(), 0, "The basket should be empty")

        basket_add_url = reverse("basket:add", args=(1,))
        body = {"quantity": 3}

        response = self.client.post(basket_add_url, body)
        self.assertEqual(
            response.status_code, 302, "Adding an item to a basket should redirect."
        )

    def test_shipping_methods(self):
        body = {
            "country": "http://testserver/checkout/api/countries/NL/",
            "first_name": "dsfsdf",
            "house_number": "",
            "last_name": "sdfsdfsd",
            "line1": "Doctor Coppesstraat 29",
            "line2": "",
            "line3": "",
            "line4": "sdfsd",
            "notes": "",
            "phone_number": "",
            "postcode": "7523EJ",
            "state": "",
            "title": "",
        }
        url = api_reverse("checkoutapi:api-basket-shipping-methods", format="html")
        response = self.client.post(
            url, body, content_type="application/json; charset=UTF-8"
        )
        self.assertEqual(response.status_code, 200)

    def test_total_call(self):
        self.fill_basket()

        body = {
            "shipping_address": {
                "country": "http://testserver/checkout/api/countries/NL/"
            },
            "shipping_method_code": "fedex",
        }

        url = reverse("checkoutapi:api-basket-total")
        response = self.client.post(
            url, body, content_type="application/json; charset=UTF-8"
        )
        self.assertEqual(response.status_code, 200)


class CheckoutSerializerTest(CheckoutApiTest):
    def get_request(self, user):
        request = factory.post("/checkout/")
        request.user = user
        if api_settings.DEFAULT_VERSIONING_CLASS is not None:
            versioning_class = api_settings.DEFAULT_VERSIONING_CLASS
            request.version = "checkoutapi"
            request.versioning_scheme = versioning_class()
        request.site = get_wagtailsite("example.com")
        return request

    def test_fill_basket(self):
        self.fill_basket()

    def test_shipping(self):
        self.assertEqual(ShippingMethod.objects.count(), 1)

    def get_data(self):
        return {
            "basket": "http://testserver/checkout/api/baskets/1/",
            "shipping_address": {
                "title": "Mr",
                "first_name": "Henk",
                "last_name": "Bertenbrau",
                "line1": "Klooiensloot",
                "house_number": "23",
                "line2": "Achter het hek",
                "line3": "2de gebbouw",
                "line4": "Arnhem",
                "state": "Gelderland",
                "postcode": "6811AL",
                "country": "http://testserver/checkout/api/countries/NL/",
                "phone_number": "0612345678",
                "notes": "",
            },
            "shipping_method_code": "fedex",
            "shipping_charge": {
                "currency": "EUR",
                "incl_tax": "5.95",
                "excl_tax": "4.9174",
                "tax": "1.0326",
            },
            "guest_email": "henk@lol.com",
        }

    def get_bare_data(self):
        return {
            "basket": "http://testserver/checkout/api/baskets/1/",
            "shipping_address": {
                "country": "http://testserver/checkout/api/countries/NL/"
            },
            "shipping_method_code": "fedex",
        }

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUTAPI_OCYAN_OPTIONS)
    def test_no_surcharges_anonymous(self):
        self.test_fill_basket()

        with self.settings(OSCAR_ALLOW_ANON_CHECKOUT=True):
            ser = serializers.CheckoutSerializer(
                data=self.get_data(),
                context={"request": self.get_request(AnonymousUser())},
            )
            self.assertTrue(ser.is_valid())
            order = ser.save()
            self.assertEqual(order.surcharges.count(), 0)
            self.assertEqual(order.total_incl_tax, D("305.95"))

    @mock.patch(
        "ocyan.plugin.oscar_checkout.checkoutapi.serializers.SurchargeApplicator",
        TestSurchargeApplicator,
    )
    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUTAPI_OCYAN_OPTIONS)
    def test_surcharges_anonymous(self):
        self.test_fill_basket()

        with self.settings(OSCAR_ALLOW_ANON_CHECKOUT=True):
            ser = serializers.CheckoutSerializer(
                data=self.get_data(),
                context={"request": self.get_request(AnonymousUser())},
            )
            self.assertTrue(ser.is_valid())
            validated_data = ser.validated_data
            order = ser.create(validated_data)
            self.assertEqual(order.surcharges.count(), 2)
            self.assertEqual(order.total_incl_tax, D("356.55"))

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUTAPI_OCYAN_OPTIONS)
    def test_international_nl_phone_number(self):
        self.test_fill_basket()

        with self.settings(OSCAR_ALLOW_ANON_CHECKOUT=True):
            data = self.get_data()
            data["shipping_address"]["phone_number"] = "+31612345678"
            ser = serializers.CheckoutSerializer(
                data=data, context={"request": self.get_request(AnonymousUser())}
            )
            self.assertTrue(ser.is_valid())

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUTAPI_OCYAN_OPTIONS)
    def test_localized_nl_phone_number(self):
        self.test_fill_basket()

        with self.settings(OSCAR_ALLOW_ANON_CHECKOUT=True):
            data = self.get_data()
            data["shipping_address"]["phone_number"] = "0612345678"
            ser = serializers.CheckoutSerializer(
                data=data, context={"request": self.get_request(AnonymousUser())}
            )
            self.assertTrue(ser.is_valid())

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUTAPI_OCYAN_OPTIONS)
    def test_wrong_localized_nl_phone_number(self):
        self.test_fill_basket()
        with self.settings(OSCAR_ALLOW_ANON_CHECKOUT=True):
            data = self.get_data()
            data["shipping_address"]["phone_number"] = "0992345678"
            ser = serializers.CheckoutSerializer(
                data=data, context={"request": self.get_request(AnonymousUser())}
            )
            self.assertFalse(ser.is_valid())

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUTAPI_OCYAN_OPTIONS)
    def test_localized_fi_phone_number(self):
        self.test_fill_basket()

        with self.settings(OSCAR_ALLOW_ANON_CHECKOUT=True):
            data = self.get_data()
            data["shipping_address"]["country"] = (
                "http://testserver/checkout/api/countries/FI/"
            )
            data["shipping_address"]["phone_number"] = "0912345678"
            ser = serializers.CheckoutSerializer(
                data=data, context={"request": self.get_request(AnonymousUser())}
            )
            self.assertTrue(ser.is_valid())

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUTAPI_OCYAN_OPTIONS)
    def test_localized_wrong_fi_phone_number(self):
        self.test_fill_basket()

        with self.settings(OSCAR_ALLOW_ANON_CHECKOUT=True):
            data = self.get_data()
            data["shipping_address"]["country"] = (
                "http://testserver/checkout/api/countries/FI/"
            )
            data["shipping_address"]["phone_number"] = "264611234567"
            ser = serializers.CheckoutSerializer(
                data=data, context={"request": self.get_request(AnonymousUser())}
            )
            self.assertFalse(ser.is_valid())

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUTAPI_OCYAN_OPTIONS)
    def test_phone_number_field_still_works(self):
        self.test_fill_basket()

        with self.settings(OSCAR_ALLOW_ANON_CHECKOUT=True):
            data = self.get_data()
            data["shipping_address"]["phone_number"] = PhoneNumber(
                country_code=31,
                national_number=624352007,
                extension=None,
                italian_leading_zero=None,
                number_of_leading_zeros=None,
                country_code_source=20,
                preferred_domestic_carrier_code=None,
            )
            ser = serializers.CheckoutSerializer(
                data=data, context={"request": self.get_request(AnonymousUser())}
            )
            self.assertTrue(ser.is_valid())

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUTAPI_OCYAN_OPTIONS)
    def test_bare_minimum_data(self):
        self.test_fill_basket()

        with self.settings(OSCAR_ALLOW_ANON_CHECKOUT=True):
            data = self.get_bare_data()
            ser = serializers.CheckoutSerializer(
                data=data,
                context={"request": self.get_request(AnonymousUser())},
                partial=True,
            )
            self.assertTrue(ser.is_valid())


@mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUTAPI_OCYAN_OPTIONS)
class TestPaymentMethodView(CheckoutApiTest):
    fixtures = [
        "checkout/auth",
        "checkout/catalogue",
        "checkout/shipping",
        "checkout/paymentmethod",
    ]

    def test_payment_method(self):
        self.fill_basket()
        url = reverse("checkoutapi:api-basket-payment-methods")
        response = self.client.get(url)
        self.assertListEqual(
            response.json(),
            [["dummy:payment", "Dummy betaling"]],
            "The dummy payment should be available.",
        )

        url = reverse("checkoutapi:api-basket-total")
        response = self.client.post(url, content_type="application/json")
        data = response.json()
        self.assertEqual(
            data["total"]["incl_tax"], "305.9500", "The total price should be 305.95."
        )

    def test_payment_method_minimum(self):
        pm = PaymentMethod.objects.get()
        pm.min_amount = D(100)
        pm.save()
        self.test_payment_method()

        pm.min_amount = D(400)
        pm.save()
        url = reverse("checkoutapi:api-basket-payment-methods")
        response = self.client.get(url)
        self.assertListEqual(
            response.json(),
            [],
            "The dummy payment should not be available, because the minimum amount restriction is not met.",
        )

    def test_payment_method_maximum(self):
        pm = PaymentMethod.objects.get()
        pm.max_amount = D(400)
        pm.save()
        self.test_payment_method()

        pm.max_amount = D(100)
        pm.save()
        url = reverse("checkoutapi:api-basket-payment-methods")
        response = self.client.get(url)
        self.assertListEqual(
            response.json(),
            [],
            "The dummy payment should not be available, because the maximum amount restriction is not met.",
        )

    def test_payment_method_maximum_and_minimum(self):
        pm = PaymentMethod.objects.get()
        pm.max_amount = D(400)
        pm.min_amount = D(100)
        pm.save()
        self.test_payment_method()

        pm.min_amount = D(320)
        pm.max_amount = D(350)
        pm.save()
        url = reverse("checkoutapi:api-basket-payment-methods")
        response = self.client.get(url)
        self.assertListEqual(
            response.json(),
            [],
            "The dummy payment should not be available, because the minimum amount restriction is not met.",
        )


@mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUTAPI_OCYAN_OPTIONS)
class TestOrderTotalView(CheckoutApiTest):
    fixtures = [
        "checkout/auth",
        "checkout/catalogue",
        "checkout/shipping",
        "checkout/paymentmethod",
    ]

    def test_no_basket(self):
        data = {
            "guest_email": "cura6b@gmail.com",
            "payment_method": "dummy:payment",
            "shipping_address": {
                "country": "http://testserver/checkout/api/countries/NL/",
                "first_name": "Rob",
                "house_number": "229-123",
                "last_name": "van den Berg",
                "line1": "Nieuwe Veenendaalseweg",
                "line4": "Rhenen",
                "phone_number": "0620306524",
                "postcode": "3911MJ",
                "title": "Mr",
            },
            "shipping_method_code": "postnl",
        }
        url = reverse("checkoutapi:api-basket-total")
        response = self.client.post(
            url, json=data, content_type="application/json; charset=UTF-8"
        )
        self.assertEqual(
            response.status_code,
            406,
            "Order total should not be available for requests with an unsaved basket",
        )



# -*- coding: utf-8 -*-
from unittest import mock

from django.conf import settings
from django.contrib.auth import get_user_model
from django.test import TestCase
from django.urls import reverse
from django.utils.translation import gettext

from oscar.core.loading import get_model

import ocyan.core.fender
from ocyan.core.utils import merge_dicts

Basket = get_model("basket", "Basket")
Product = get_model("catalogue", "Product")
User = get_user_model()

ocyan_settings = ocyan.core.fender.config.settings
CHECKOUT_OCYAN_OPTIONS = merge_dicts(
    ocyan_settings, {"shipping": {"enable_charged_shipping": True}}
)


class CheckoutTest(TestCase):
    fixtures = [
        "checkout/auth",
        "checkout/catalogue",
        "checkout/shipping",
        "checkout/sites",
    ]

    def test_catalogue_initialized(self):
        "The catalogue should be initialized"
        self.assertEqual(Product.objects.count(), 2)
        self.assertEqual(User.objects.count(), 2)

    def test_checkout_empty_basket(self):
        "Checkout with empty basket should be disabled"
        if not settings.OSCAR_ALLOW_ANON_CHECKOUT:
            henk = User.objects.get(username="henk")
            self.client.force_login(henk)

        if not settings.SHOW_BASKET_POPUP_MODAL:
            response = self.client.get(reverse("checkout:index"))
            self.assertEqual(
                response.status_code,
                200,
                "The checkout page should work just fine even without a basket",
            )
            self.assertContains(
                response, gettext("Your shopping basket appears to be empty")
            )

    def test_checkout_thank_you_404_or_302(self):
        if not settings.OSCAR_ALLOW_ANON_CHECKOUT:
            henk = User.objects.get(username="henk")
            self.client.force_login(henk)

        response = self.client.get(reverse("checkout:thank-you"))
        self.assertIn(
            response.status_code,
            [404, 302],
            "There should be no thank you page if there is not order",
        )

    @mock.patch("sorl.thumbnail.templatetags.thumbnail.ThumbnailNodeBase.render")
    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUT_OCYAN_OPTIONS)
    def test_checkout_with_filled_basket(self, render):
        "The checkout page should work fine with items in the basket"
        render.return_value = ""

        henk = User.objects.get(username="henk")
        self.client.force_login(henk)
        self.assertEqual(Basket.objects.count(), 0, "The basket should be empty")

        basket_add_url = reverse("basket:add", args=(1,))
        body = {"quantity": 3}
        response = self.client.post(basket_add_url, body)
        self.assertEqual(
            response.status_code, 302, "Adding an item to a basket should redirect."
        )

        if not settings.SHOW_BASKET_POPUP_MODAL:
            self.assertEqual(
                response["location"],
                reverse("basket:summary"),
                "Adding item to basket should redirect to basket",
            )

            response = self.client.get(reverse("checkout:index"))
            self.assertEqual(
                response.status_code, 200, "The checkout page should work just fine."
            )
        else:
            success_url = reverse("basket:summary")
            product_pk = 1
            expected_redirect_url = f"{success_url}#prod_{product_pk}"

            self.assertIn(
                expected_redirect_url,
                response["location"],
                "When the basket popup modal is shown, the redirect URL should contain the product reference.",
            )

            self.assertEqual(
                self.client.session.get("added_product"),
                product_pk,
                "The session should store the added product's primary key.",
            )
            self.assertEqual(
                self.client.session.get("added_quantity"),
                str(3),
                "The session should store the correct added quantity.",
            )

    @mock.patch.object(ocyan.core.fender.config, "settings", CHECKOUT_OCYAN_OPTIONS)
    def test_checkout_country_billing_field(self):
        henk = User.objects.get(username="henk")
        self.client.force_login(henk)
        self.assertEqual(Basket.objects.count(), 0, "The basket should be empty")

        basket_add_url = reverse("basket:add", args=(1,))
        body = {"quantity": 3}

        response = self.client.post(basket_add_url, body)
        if not settings.SHOW_BASKET_POPUP_MODAL:
            self.assertEqual(
                response.status_code, 302, "Adding an item to a basket should redirect."
            )
            self.assertEqual(
                response["location"],
                reverse("basket:summary"),
                "Adding item to basket should redirect to basket",
            )
            response = self.client.get(reverse("checkout:index"))
            self.assertEqual(
                response.status_code, 200, "The checkout page should work just fine."
            )
            self.assertContains(
                response, '<div class="form-group id_billing_address_form-country ">'
            )
        else:
            success_url = reverse("basket:summary")
            product_pk = 1
            expected_redirect_url = f"{success_url}#prod_{product_pk}"

            self.assertIn(
                expected_redirect_url,
                response["location"],
                "When the basket popup modal is shown, the redirect URL should contain the product reference.",
            )

            self.assertEqual(
                self.client.session.get("added_product"),
                product_pk,
                "The session should store the added product's primary key.",
            )
            self.assertEqual(
                self.client.session.get("added_quantity"),
                str(3),
                "The session should store the correct added quantity.",
            )



# Generated by Django 3.2.19 on 2023-07-10 10:29

from django.db import migrations, models

from oscar.core.loading import get_classes

get_default_tax_rate, ALL_TAX_RATES = get_classes(
    "taxes.taxrate",
    [
        "get_default_tax_rate",
        "ALL_TAX_RATES",
    ],
)


class Migration(migrations.Migration):

    dependencies = [
        ('checkout', '0014_auto_20220607_1354'),
    ]

    operations = [
        migrations.AddField(
            model_name='fixedsurcharge',
            name='price',
            field=models.DecimalField(decimal_places=2, default=0, max_digits=12),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='fixedsurcharge',
            name='vat_rate',
            field=models.CharField(choices=ALL_TAX_RATES, default=get_default_tax_rate(), max_length=5, null=True, verbose_name='VAT rate'),
        ),
        migrations.AddField(
            model_name='percentagesurcharge',
            name='vat_rate',
            field=models.CharField(choices=ALL_TAX_RATES, default=get_default_tax_rate(), max_length=5, null=True, verbose_name='VAT rate'),
        ),
    ]



from wagtail.models import Site
from wagtail.rich_text import expand_db_html

from ocyan.plugin.wagtail.models import OcyanSettings


class DeliveryTime(object):
    def __init__(self, delivery_time_text, delivery_time_days):
        self.delivery_time_text = delivery_time_text
        self.delivery_time_days = delivery_time_days

    def __str__(self):
        if self.delivery_time_text is not None:
            return expand_db_html(self.delivery_time_text)
        return ""

    def __bool__(self):
        return bool(self.delivery_time_days) and self.delivery_time_text is not None

    @classmethod
    def delivery_time(cls, basket, request=None):
        if request is not None:
            site = Site.find_for_request(request)
        else:
            site = Site.objects.get(is_default_site=True)

        settings = OcyanSettings.for_site(site)

        return cls(settings.delivery_time_text, settings.delivery_time_days)



from ocyan.core import plugins as core_plugin


class PaymentPluginConfig(core_plugin.OcyanPluginConfig):
    def contribute_to_payment_methods(self, payment_methods):
        return payment_methods

    def contribute_to_direct_payment_methods(self, direct_payment_methods):
        return direct_payment_methods


def get_payment_plugins():
    return core_plugin.get_plugins(cls=PaymentPluginConfig)


def get_payment_method_choices():
    payment_plugins = get_payment_plugins()
    return payment_plugins.pipe("contribute_to_payment_methods", [])


def get_direct_payment_method_choices():
    payment_plugins = get_payment_plugins()
    return payment_plugins.pipe("contribute_to_direct_payment_methods", {})



# pylint: disable=W0223
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from phonenumber_field.modelfields import PhoneNumberField
from rest_framework import exceptions, serializers

from oscar.core import prices
from oscar.core.loading import get_class, get_model
from oscar.templatetags.currency_filters import currency

from oscarapi.basket.operations import assign_basket_strategy
from oscarapi.serializers.checkout import (
    CheckoutSerializer as BaseCheckoutSerializer,
)
from oscarapi.serializers.checkout import (
    PriceSerializer as BasePriceSerializer,
)
from oscarapi.serializers.checkout import (
    ShippingAddressSerializer as BaseShippingAddressSerializer,
)
from oscarapi.serializers.checkout import (
    ShippingMethodSerializer as BaseShippingMethodSerializer,
)
from oscarapi.serializers.fields import TaxIncludedDecimalField

from ocyan.main.loading import get_ocyan_class

SurchargeApplicator = get_class("checkout.applicator", "SurchargeApplicator")
OrderTotalCalculator = get_class("checkout.calculators", "OrderTotalCalculator")
PhoneNumberFormContext = get_ocyan_class(
    "checkoutapi.phone_number", "PhoneNumberFormContext"
)

ShippingAddress = get_model("order", "ShippingAddress")
BillingAddress = get_model("order", "BillingAddress")


class PriceSerializer(BasePriceSerializer):
    excl_tax = serializers.DecimalField(decimal_places=4, max_digits=12, required=True)
    incl_tax = TaxIncludedDecimalField(
        excl_tax_field="excl_tax", decimal_places=4, max_digits=12, required=False
    )
    tax = TaxIncludedDecimalField(
        excl_tax_value="0.00", decimal_places=4, max_digits=12, required=False
    )
    formatted_excl_tax = serializers.SerializerMethodField()

    def get_formatted_excl_tax(self, obj):
        return currency(obj.excl_tax, obj.currency)


class PaymentMethodSerializer(serializers.Serializer):
    country = serializers.CharField(max_length=2, required=False, allow_blank=True)
    key = serializers.CharField(required=False, allow_blank=True)
    label = serializers.CharField(required=False, allow_blank=True)
    snippet = serializers.CharField(allow_blank=True, required=False)


class ShippingMethodSerializer(BaseShippingMethodSerializer):
    def calculate_price(self, obj):
        price = obj.calculate(self.context.get("basket"))
        return PriceSerializer(price).data


class ShippingAddressSerializer(BaseShippingAddressSerializer):
    line1 = serializers.CharField(max_length=255, required=False, allow_blank=True)
    phone_number = serializers.ModelField(
        model_field=PhoneNumberField(blank=True), validators=[]
    )

    def validate(self, attrs):
        data = super().validate(attrs)
        phonectx = PhoneNumberFormContext(data)
        data["phone_number"] = phonectx.clean_phone_number_field("phone_number")
        return data

    def add_error(self, field_name, error):
        raise serializers.ValidationError(error)


class ShippingCountrySerializer(ShippingAddressSerializer):
    # we only care about the country, the rest is nice, but optional
    house_number = serializers.CharField(
        required=False, allow_null=True, allow_blank=True, max_length=255
    )


class OrderTotalSerializer(serializers.Serializer):
    total = PriceSerializer(many=False)
    shipping_charge = PriceSerializer(many=False)
    total_incl_tax_excl_discounts = serializers.DecimalField(
        decimal_places=2, max_digits=12, source="basket.total_incl_tax_excl_discounts"
    )
    price_excl_tax_excl_discounts = serializers.DecimalField(
        decimal_places=2, max_digits=12, source="basket.total_excl_tax_excl_discounts"
    )
    snippet = serializers.CharField(allow_blank=True, required=False)


class CheckoutSerializer(BaseCheckoutSerializer):
    shipping_address = ShippingAddressSerializer(many=False, required=False)
    payment_method = serializers.CharField(required=False)
    shipping_charge = PriceSerializer(many=False, required=False)

    @property
    def request(self):
        return self.context["request"]

    def validate(self, attrs):
        request = self.request

        if request.user.is_anonymous:
            if not settings.OSCAR_ALLOW_ANON_CHECKOUT:
                message = _("Anonymous checkout forbidden")
                raise serializers.ValidationError(message)

            if not self.partial and not attrs.get("guest_email"):
                # Always require the guest email field if the user is anonymous
                message = _("Guest email is required for anonymous checkouts")
                raise serializers.ValidationError(message)
        else:
            if "guest_email" in attrs:
                # Don't store guest_email field if the user is authenticated
                del attrs["guest_email"]

        basket = attrs.get("basket")
        shipping_address = attrs.get("shipping_address")

        if (  # this code is only needed for vat plugin
            shipping_address is not None and "country" in shipping_address
        ):
            country = shipping_address["country"]
            request.META["shipping_country"] = country.pk

        basket = assign_basket_strategy(basket, request)
        if basket.num_items <= 0:
            message = _("Cannot checkout with empty basket")
            raise serializers.ValidationError(message)

        shipping_method = self._shipping_method(
            request, basket, attrs.get("shipping_method_code"), shipping_address
        )

        shipping_charge = shipping_method.calculate(basket)
        posted_shipping_charge = attrs.get("shipping_charge")

        if posted_shipping_charge is not None:
            posted_shipping_charge = prices.Price(**posted_shipping_charge)
            # test submitted data.
            if not posted_shipping_charge == shipping_charge:
                message = _(
                    "Shipping price incorrect %s != %s"
                    % (posted_shipping_charge, shipping_charge)
                )
                raise serializers.ValidationError(message)

        posted_total = attrs.get("total")
        surcharges = SurchargeApplicator(request, attrs).get_applicable_surcharges(
            basket, shipping_charge=shipping_charge
        )
        total = OrderTotalCalculator().calculate(basket, shipping_charge, surcharges)
        if posted_total is not None:
            if posted_total != total.incl_tax:
                message = _("Total incorrect %s != %s" % (posted_total, total.incl_tax))
                raise serializers.ValidationError(message)

        # update attrs with validated data.
        attrs["total"] = total
        attrs["shipping_method"] = shipping_method
        attrs["shipping_charge"] = shipping_charge
        attrs["basket"] = basket
        attrs["surcharges"] = surcharges
        return attrs

    def create(self, validated_data):
        try:
            basket = validated_data.get("basket")
            order_number = self.generate_order_number(basket)
            request = self.request

            if "shipping_address" in validated_data:
                shipping_address = ShippingAddress(**validated_data["shipping_address"])
            else:
                shipping_address = None

            if "billing_address" in validated_data:
                billing_address = BillingAddress(**validated_data["billing_address"])
            else:
                billing_address = None

            return self.place_order(
                order_number=order_number,
                user=request.user,
                basket=basket,
                shipping_address=shipping_address,
                shipping_method=validated_data.get("shipping_method"),
                shipping_charge=validated_data.get("shipping_charge"),
                surcharges=validated_data.get("surcharges"),
                billing_address=billing_address,
                order_total=validated_data.get("total"),
                guest_email=validated_data.get("guest_email") or "",
            )
        except ValueError as e:
            raise exceptions.NotAcceptable(str(e))



# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from ocyan.core.fender import config

from ocyan.plugin.oscar_checkout.checkout.models import (
    FixedSurcharge,
    PaymentMethod,
    PercentageSurcharge,
)

INVOICE_ACTIVE = config.get("oscar_communications", "invoice", False)


# Register your models here.
class PaymentMethodAdmin(admin.ModelAdmin):
    fieldsets = (
        (
            None,
            {
                "fields": (
                    ("label", "key", "send_invoice")
                    if INVOICE_ACTIVE
                    else ("label", "key")
                )
            },
        ),
        (_("Availability"), {"fields": ("country",)}),
        (_("Limits"), {"fields": ("min_amount", "max_amount")}),
        (
            _("Permissions"),
            {"classes": ("collapse",), "fields": ("requires_permission",)},
        ),
        (_("Ordering"), {"fields": ("ordering",)}),
    )

    filter_horizontal = ["country"]


class SurchargeAdmin(admin.ModelAdmin):
    filter_horizontal = ["country", "payment_method"]


admin.site.register(PaymentMethod, PaymentMethodAdmin)
admin.site.register(FixedSurcharge, SurchargeAdmin)
admin.site.register(PercentageSurcharge, SurchargeAdmin)
